@@ -20,7 +20,7 @@
 #ifndef DEFS_H
 #define DEFS_H
 #include <stdio.h>
-#include <iostream.h>
+#include "iostream.h"
 #include <cassert>
 #include <limits.h>
 #include <float.h>
@@ -39,8 +39,8 @@ inline void zaparr(T & x)
      delete [] x;
      x = NULL;
 }
-static char* release_no="0.41alpha";
-static char* release="Freecycle 0.41alpha";
+static char* release_no="0.51alpha";
+static char* release="Freecycle 0.51alpha";
 static const int RTTI_BEATLINE=1001;
 static const int RTTI_BPMLINE=1002;
 static const int RTTI_LOOPMARKER=1003;
@@ -52,6 +52,14 @@ static const int RTTI_SPLINELINE=1008;
 static const int RTTI_LOCKEDBEATRECT=1009;
 static const int RTTI_LADSPAVALUERECT=1010;
 static const int RTTI_KEYBOARDRECT=1011;
+static const int RTTI_PITCHSHIFTRECT=1012;
+
+
+static const double BEAT_DISTANCE_TRESHOLD=128;
+
+static const int MIDI_INPUT_EVENT=2000;
+static const int PLAY_POSITION_CHANGED_EVENT=2001;
+
 
 #endif
 
