@@ -102,13 +102,14 @@ void EffectPortFrame::resizeEvent(QResizeEvent* ev)
     //updateGeometry();
 }
 
-void EffectPortFrame::drawValueLines(){
+void EffectPortFrame::drawValueLines(bool updateCanvas){
     float max=port->getUpperBound();
     float min=port->getLowerBound();
     float diff=max-min;
     int count=0;
     if(!linesCreated){
-        for (double i=0;i<port->getNbFrames();i+=1024){
+        int reminder=port->getNbFrames() % 1024;
+        for (double i=0;i<port->getNbFrames()-reminder;i+=1024){
                 ValueLine* line=new ValueLine(canvas,(long)i,(float*)port->getVOTPointer((int)i/1024));
                 int drawValue=(int)(visibleHeight()-((line->getValue()-min)*visibleHeight())/diff);
                 line->setPen(darkGray);
@@ -135,7 +136,7 @@ void EffectPortFrame::drawValueLines(){
             }
         }
     }
-    canvas->update();
+    if(updateCanvas) canvas->update();
     //cout<<"VALUELINES_NB= "<<count<<endl;
 }
 
@@ -153,16 +154,16 @@ void EffectPortFrame::closeEvent( QCloseEvent* ce ){
  */
 void EffectPortFrame::setZoomRatio(long _zr)
 {
-    zoomRatio=_zr;
-    canvas->resize((int)((double)port->getNbFrames()/(double)zoomRatio),visibleHeight());
-    cout<<"Effect frame canvas resized\n";
-    drawValueLines();
-    drawBeatlines();
-    polyLine->setZoomRatio(zoomRatio);
-    polyLine->updateSize(canvas->size());
-    spline->setZoomRatio(zoomRatio);
-    spline->updateSize(canvas->size());
-    canvas->update();
+        zoomRatio=_zr;
+        canvas->resize((int)((double)port->getNbFrames()/(double)zoomRatio),visibleHeight());
+        //cout<<"Effect frame canvas resized\n";
+        drawValueLines(false);
+        drawBeatlines(false);
+        polyLine->setZoomRatio(zoomRatio);
+        polyLine->updateSize(canvas->size());
+        spline->setZoomRatio(zoomRatio);
+        spline->updateSize(canvas->size());
+        //canvas->update();
 }
 
 
@@ -534,9 +535,9 @@ EffectPortFrame::DrawMode EffectPortFrame::getDrawMode()
 
 
 /*!
-    \fn EffectPortFrame::beatLinesChanged(QValueList <long> beatlines)
+    \fn EffectPortFrame::beatLinesChanged(QValueList <beatline_data> beatlines)
  */
-void EffectPortFrame::beatLinesChanged(QValueList <long> _beatlines)
+void EffectPortFrame::beatLinesChanged(QValueList <beatline_data> _beatlines)
 {
     beatlines=_beatlines;
     drawBeatlines();
@@ -545,9 +546,9 @@ void EffectPortFrame::beatLinesChanged(QValueList <long> _beatlines)
 
 
 /*!
-    \fn EffectPortFrame::lockedBeatLinesChanged(QValueList <long> beatlines)
+    \fn EffectPortFrame::lockedBeatLinesChanged(QValueList <beatline_data> beatlines)
  */
-void EffectPortFrame::lockedBeatLinesChanged(QValueList <long> _beatlines)
+void EffectPortFrame::lockedBeatLinesChanged(QValueList <beatline_data> _beatlines)
 {
     lockedBeatlines=_beatlines;
     drawBeatlines();
@@ -558,7 +559,7 @@ void EffectPortFrame::lockedBeatLinesChanged(QValueList <long> _beatlines)
 /*!
     \fn EffectPortFrame::drawBeatlines()
  */
-void EffectPortFrame::drawBeatlines()
+void EffectPortFrame::drawBeatlines(bool updateCanvas)
 {
     removeBeatlines();
     /*
@@ -572,16 +573,16 @@ void EffectPortFrame::drawBeatlines()
         }
     }
     */
-    cout<<"Drawing "<<lockedBeatlines.count()<<" locked beatlines\n";
+    //cout<<"Drawing "<<lockedBeatlines.count()<<" locked beatlines\n";
     if(lockedBeatlines.count()!=0){
         for (int i=0;i<lockedBeatlines.count();i++){
-            BeatLine* line=new BeatLine(canvas,lockedBeatlines[i]);
+            BeatLine* line=new BeatLine(canvas,lockedBeatlines[i].position);
             line->lock();
             line->setZ(10);
             line->setDrawHandles(0);
             line->setX(line->getBeatPos() / zoomRatio);
             line->setPen(green);
-            int x=lockedBeatlines[i]/zoomRatio;
+            int x=lockedBeatlines[i].position/zoomRatio;
             line->setPoints(x,0,x,visibleHeight());
             line->show();
         }
