@@ -25,7 +25,11 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
  : QHBox(parent, name)
 {
 
-    setSpacing(0);
+    e_width=0;
+    setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Minimum);
+    setMinimumSize(360,50);
+    setFrameStyle(QFrame::Panel | QFrame::Raised);
+    setSpacing(5);
     setMargin(0);
 
     inHolder=false;
@@ -36,8 +40,7 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
     font.setPixelSize(11);
 
     QVBox* onoffbox=new QVBox(this);
-    onoffbox->setFixedWidth(20);
-    onoffbox->setFixedHeight(48);
+    
     onoffbox->setSpacing(0);
     onoffbox->setMargin(0);
 
@@ -46,18 +49,14 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
     onoff->setFont(font);
     onoff->setToggleButton(TRUE);
     onoff->setOn(FALSE);
-    onoff->setFixedHeight(16);
-    onoff->setFixedWidth(20);
-
+    
     QWidget* empty1=new QWidget(onoffbox);
-    empty1->setFixedHeight(32);
-
+    
     connect(onoff,SIGNAL(toggled(bool)),this,SLOT(onoffChanged(bool )));
 
     font.setPixelSize(8);
     displayEnvelope=new DisplayEnvelopeWidget(this);
     displayEnvelope->setEnvData(envData);
-    displayEnvelope->setFixedHeight(48);
     displayEnvelope->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
     displayEnvelope->setLineWidth(2);
 
@@ -65,16 +64,13 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
             this, SIGNAL(envelopeChanged(EnvelopeData* )));
 
     QVBox* durationBox=new QVBox(this);
-    durationBox->setFixedWidth(30);
-
+    
     QLabel* durNameLabel=new QLabel(durationBox);
     durNameLabel->setText("dur:");
     durNameLabel->setFont(font);
     durNameLabel->setAlignment(AlignHCenter);
 
     envDurationWidget=new PotentiometerWidget(durationBox);
-    envDurationWidget->setFixedHeight(30);
-    envDurationWidget->setFixedWidth(30);
     envDurationWidget->setMinValue(100);
     envDurationWidget->setMaxValue(3000);
     envDurationWidget->setValue(100);
@@ -93,7 +89,7 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
     connect(envDurationWidget,SIGNAL(dialReleased()),this,SLOT(durationChanged()));
 
     QVBox* resolutionBox=new QVBox(this);
-    resolutionBox->setFixedWidth(30);
+    
 
     QLabel* resNameLabel=new QLabel(resolutionBox);
     resNameLabel->setText("res:");
@@ -101,8 +97,6 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
     resNameLabel->setAlignment(AlignHCenter);
 
     envResolutionWidget=new PotentiometerWidget(resolutionBox);
-    envResolutionWidget->setFixedHeight(30);
-    envResolutionWidget->setFixedWidth(30);
     envResolutionWidget->setMinValue(10);
     envResolutionWidget->setMaxValue(1024);
     envResolutionWidget->setValue(100);
@@ -127,14 +121,12 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
     optionsBG->setFrameShape(NoFrame);
     optionsBG->setFixedWidth(0);
     optionsBG->setFixedHeight(0);
+    
 
     QVBox* optionsbox1=new QVBox(this);
     QVBox* optionsbox2=new QVBox(this);
 
-    optionsbox1->setFixedWidth(30);
-    optionsbox2->setFixedWidth(30);
-    optionsbox1->setFixedHeight(48);
-    optionsbox2->setFixedHeight(48);
+    
 
     optionsbox1->setFrameShape(NoFrame);
     optionsbox1->setSpacing(0);
@@ -152,56 +144,80 @@ EnvelopeControllerWidget::EnvelopeControllerWidget(QWidget *parent, const char *
     end->setText("end");
     end->setFont(font);
     end->setToggleButton(TRUE);
-    end->setFixedHeight(16);
-    end->setFixedWidth(30);
+    
     optionsBG->insert(end);
     QPushButton* cont=new QPushButton(optionsbox1);
     cont->setText("cont");
     cont->setFont(font);
     cont->setToggleButton(TRUE);
-    cont->setFixedHeight(16);
-    cont->setFixedWidth(30);
+    
     optionsBG->insert(cont);
     QPushButton* next=new QPushButton(optionsbox1);
     next->setText("next");
     next->setFont(font);
     next->setToggleButton(TRUE);
-    next->setFixedHeight(16);
-    next->setFixedWidth(30);
+    
     optionsBG->insert(next);
 
     QPushButton* stretch=new QPushButton(optionsbox2);
     stretch->setText("str");
     stretch->setFont(font);
     stretch->setToggleButton(TRUE);
-    stretch->setFixedHeight(16);
-    stretch->setFixedWidth(30);
+    
     optionsBG->insert(stretch);
     QPushButton* stretch2=new QPushButton(optionsbox2);
     stretch2->setText("str2");
     stretch2->setFont(font);
     stretch2->setToggleButton(TRUE);
-    stretch2->setFixedHeight(16);
-    stretch2->setFixedWidth(30);
+    
     optionsBG->insert(stretch2);
     QPushButton* auto1=new QPushButton(optionsbox2);
     auto1->setText("auto");
     auto1->setFont(font);
     auto1->setToggleButton(TRUE);
-    auto1->setFixedHeight(16);
-    auto1->setFixedWidth(30);
+    
     optionsBG->insert(auto1);
     cont->setOn(TRUE);
 
 
     connect(optionsBG,SIGNAL(clicked(int)),this,SLOT(endModeChanged(int)));
+    
+    #ifndef Q_OS_MACX
+    empty1->setFixedSize(0,32);
+    onoff->setFixedSize(20,16);
+    onoffbox->setFixedSize(20,48);
+    end->setFixedSize(30,16);
+    cont->setFixedSize(30,16);
+    stretch->setFixedSize(30,16);
+    stretch2->setFixedSize(30,16);
+    auto1->setFixedSize(30,16);
+    next->setFixedSize(30,16);
+    optionsbox1->setFixedSize(30,48);
+    optionsbox2->setFixedSize(30,48);
+    #endif
+    
+    envResolutionWidget->setFixedSize(30,30);
+    resolutionBox->setFixedWidth(30);
+    envDurationWidget->setFixedSize(30,30);
+    durationBox->setFixedWidth(30);
+    displayEnvelope->setFixedHeight(48);
+    displayEnvelope->setMinimumWidth(180);
+    
+    e_width=onoffbox->width() + displayEnvelope->width() + durationBox->width() + resolutionBox->width() +end->width()*2;
+    
 }
 
 
+
+
 EnvelopeControllerWidget::~EnvelopeControllerWidget()
 {
 }
 
+QSize EnvelopeControllerWidget::sizeHint () const{
+    return QSize(e_width,50);
+}
+
 
 
 
