@@ -45,9 +45,6 @@
 #include <qvbox.h>
 #include <qcanvas.h>
 #include <iostream.h>
-#include 	<fcntl.h>
-#include 	<sys/ioctl.h>
-#include 	<sys/soundcard.h>
 #include 	<string.h>
 #include <unistd.h>
 #include <qspinbox.h>
@@ -60,16 +57,22 @@
 Freecycle::Freecycle()
     : QMainWindow( 0, release, WDestructiveClose )
 {
+    loadingProject=FALSE;
     QDir freecycle_dir(QDir::homeDirPath().append("/.freecycle"));
     if (!freecycle_dir.exists()) freecycle_dir.mkdir(freecycle_dir.absPath());
+    helpManager=new HelpManager(freecycle_dir.path());
 
     soundManager=new SoundManager();
+    soundManager->setHasEnvelope(TRUE);
     soundManager->loadEffects();
     fileManager=new FileManager();
     soundManager->setFileManager(fileManager);
     midiManager=new MidiManager();
     akpManager=new AKPManager();
-
+#ifdef HAS_LIBINSTPATCH
+    sf2Manager=new SF2Manager();
+    gigaManager=new GigaManager();
+#endif
     QPixmap openIcon, saveIcon, printIcon;
     openIcon = QPixmap( fileopen );
     saveIcon = QPixmap( filesave );
@@ -80,13 +83,17 @@ Freecycle::Freecycle()
     filePopupMenu->insertItem( openIcon, tr("&Open Wave..."),
 			   this, SLOT(choose()), CTRL+Key_O );
 
-    idsavedialog=filePopupMenu->insertItem( saveIcon, tr("&Save Wave"),
-			   this, SLOT(save()), CTRL+Key_S );
+    idsavedialog=filePopupMenu->insertItem( saveIcon, tr("Save Wave"),
+			   this, SLOT(save()));
 
     idsaveasdialog= filePopupMenu->insertItem( tr("Save Wave &As..."), this, SLOT(saveAs()) );
     idsavechunksdialog=filePopupMenu->insertItem( tr("Save Audio Chunks..."), this, SLOT(saveChunks()) );
-    idsaveprojectdialog=filePopupMenu->insertItem( tr("Save Project..."), this, SLOT(saveProject()) );
+    idsaveprojectasdialog=filePopupMenu->insertItem( tr("Save Project As..."), this, SLOT(saveProjectAs()) );
+    idsaveprojectdialog=filePopupMenu->insertItem( tr("&Save Project"), this, SLOT(saveProject()),CTRL+Key_S );
+    idsaveloopmidifile=filePopupMenu->insertItem( tr("Export Loop &MIDI file..."), this, 
+                                                    SLOT(saveLoopMidiFile()),CTRL+Key_M );
     idloadprojectdialog=filePopupMenu->insertItem( tr("Load Project..."), this, SLOT(loadProject()) );
+    
 
 
     filePopupMenu->insertSeparator();
@@ -146,6 +153,12 @@ Freecycle::Freecycle()
     configurationPopupMenu->insertItem(tr("Configure FFT ..."),this,SLOT(configureFFT()));
     ampDomainDialog=new AmpDomainDialog(soundManager,this);
     configurationPopupMenu->insertItem(tr("Configure Amplitude domain ..."),this,SLOT(configureAmpDomain()));
+    
+    #ifdef HAS_PORTMIDI
+        midiInput=new MidiInput();
+        midiConfigDialog=new MidiConfigDialog(midiInput,this);
+        configurationPopupMenu->insertItem(tr("Configure MIDI ..."),this,SLOT(configureMIDI()));
+    #endif
 
 
     menuBar()->insertItem( tr("&Configuration"), configurationPopupMenu,3);
@@ -154,7 +167,7 @@ Freecycle::Freecycle()
 
 
 
-    resize( 800, 600 );
+    resize( 1000, 800 );
 
 
 
@@ -166,7 +179,7 @@ Freecycle::Freecycle()
 
     //Top status bar
     topBox=new QHBox(centralLayout,"File info");
-    topBox->setSpacing(10);
+    topBox->setSpacing(5);
     topBox->setMaximumHeight(50);
     topBox->setFrameStyle(QFrame::Panel | QFrame::Raised);
     topBox->hide();
@@ -181,18 +194,18 @@ Freecycle::Freecycle()
     bpmModifDropDown=new BPMModifierDropDown(topBox);
     connect(bpmModifDropDown,SIGNAL(valueChanged()),this,SLOT(bpmModifierValueChanged()));
 
-
-    resampleButton=new QPushButton("time stretch to:",topBox,"resample button");
-
-    newBPM=new FloatSpinBox(topBox);
-    newBPM->setMinValue(300);
-    newBPM->setMaxValue(3000);
+    #ifdef HAS_SOUNDTOUCH
+        resampleButton=new QPushButton("time stretch to:",topBox,"resample button");
+        newBPM=new FloatSpinBox(topBox);
+        newBPM->setMinValue(300);
+        newBPM->setMaxValue(3000);
+    #endif
 
 
 
     //End Top status Bar
     playAndBeatLinesControls=new QHBox(centralLayout,"Play and BeatLines");
-    playAndBeatLinesControls->setSpacing(10);
+    playAndBeatLinesControls->setSpacing(0);
     playAndBeatLinesControls->setSizePolicy(QSizePolicy::Maximum,QSizePolicy::Minimum);
     playAndBeatLinesControls->hide();
     //Play buttons bar
@@ -249,7 +262,7 @@ Freecycle::Freecycle()
     tresholdSlider->setTickInterval(10);
     tresholdSlider->setLineStep(1);
     tresholdSlider->setPageStep(10);
-    tresholdSlider->setMaximumWidth(200);
+    tresholdSlider->setFixedWidth(200);
     connect(tresholdSlider, SIGNAL(sliderReleased()), SLOT(changeTreshold()) );
 
     analyseWindowDivider=new QSlider(beatLinesControl);
@@ -278,15 +291,17 @@ Freecycle::Freecycle()
     lockBeatLinesButton->setPixmap(QPixmap(qembed_findImage("encrypted-22")));
 
     envelopeControllerWidget=new EnvelopeControllerWidget(playAndBeatLinesControls);
-    envelopeControllerWidget->setFixedHeight(50);
-    envelopeControllerWidget->setFixedWidth(350);
-    envelopeControllerWidget->setFrameStyle(QFrame::Panel | QFrame::Raised);
-    envelopeControllerWidget->setSpacing(5);
-    envelopeControllerWidget->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Fixed);
     //envelopeControllerWidget->hide(); //for now..
     connect(envelopeControllerWidget,SIGNAL(envelopeChanged(EnvelopeData* )),
             soundManager,SLOT(envelopeChanged(EnvelopeData* )));
 
+    sceneFrame=new SceneFrame(playAndBeatLinesControls);
+    sceneFrame->setFixedHeight(50);
+    sceneFrame->setFixedWidth(182);
+    sceneFrame->setFrameStyle(QFrame::Panel | QFrame::Raised);
+    sceneFrame->setSpacing(0);
+    sceneFrame->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Fixed);
+    
 
     big_and_small_wave_and_effects=new MySplitter(Qt::Vertical,centralLayout);
     //big_and_small_wave_and_effects->setOpaqueResize(TRUE);
@@ -301,27 +316,36 @@ Freecycle::Freecycle()
 
     drawSoundWidget=new DrawSoundWidget(big_wave_and_env,"central wave",soundManager);
     drawSoundWidget->enableClipper(TRUE);
+    
+    connect(sceneFrame,SIGNAL(sceneSwitchedS(int)),drawSoundWidget,SLOT(sceneSwitched(int )));
+    connect(sceneFrame,SIGNAL(assignToSceneS(int)),drawSoundWidget,SLOT(assignToScene(int )));
 
     bpmLines=new QPushButton("bpm lines",topBox);
     bpmLines->setToggleButton(TRUE);
     bpmLines->setPixmap(QPixmap(qembed_findImage("kalarm")));
     bpmLines->setDisabled(TRUE);
     connect(drawSoundWidget,SIGNAL(hasLockedBeatLines(bool)),bpmLines,SLOT(setEnabled(bool)));
+    
+    #ifdef HAS_PORTMIDI
+        midiInput->addMidiMonitor(drawSoundWidget);
+    #endif
 
-    envelopeHolder=new EnvelopeHolder(big_wave_and_env);
-    envelopeHolder->setMaximumHeight(55);
-    envelopeHolder->setMinimumHeight(55);
-    envelopeHolder->hide();
+    //replacedWaveHolder=new ReplacedWaveHolder(big_wave_and_env);
+    //replacedWaveHolder->setMaximumHeight(55);
+    //replacedWaveHolder->setMinimumHeight(55);
+    //replacedWaveHolder->hide();
+    /*
     connect(drawSoundWidget,
             SIGNAL(contentsMoving(int,int)),
-            envelopeHolder,SLOT(setContentsPos(int,int)));
+            replacedWaveHolder,SLOT(setContentsPos(int,int)));
     connect(drawSoundWidget,
-            SIGNAL(receivedEnvelope(QPoint,QString)),
-            envelopeHolder,SLOT(setEnvelopeAt(QPoint,QString)));
+            SIGNAL(receivedWave(QPoint,QString)),
+            replacedWaveHolder,SLOT(setWaveAt(QPoint,QString)));
     connect(drawSoundWidget,SIGNAL(zoomChangedS(long)),
-            envelopeHolder,SLOT(setZoomRatio(long)));
+            replacedWaveHolder,SLOT(setZoomRatio(long)));
     connect(soundManager,SIGNAL(nbFramesChanged(long)),
-            envelopeHolder,SLOT(setFrames(long)));
+            replacedWaveHolder,SLOT(setFrames(long)));
+    */
     //envelopeHolder->setHeight(0);
 
     zoomBox=new QHBox(big_wave_and_zoom);
@@ -367,14 +391,15 @@ Freecycle::Freecycle()
 
 
 
-
-    connect(newBPM,
+    #ifdef HAS_SOUNDTOUCH
+        connect(newBPM,
                     SIGNAL(valueChanged(const QString &)),
                     drawSoundWidget,SLOT(computeNewBPMLines(const QString &)));
-
-
-    connect(resampleButton,
+        connect(resampleButton,
                       SIGNAL(clicked()),drawSoundWidget,SLOT(resample()));
+    #endif
+
+
     MySplitter *small_waveAndSpectrum = new MySplitter(Qt::Horizontal, big_and_small_wave );
     small_waveAndSpectrum->setOpaqueResize(TRUE);
 
@@ -400,6 +425,11 @@ Freecycle::Freecycle()
     connect(drawSoundWidget,SIGNAL(playFromPoint(long)),this,SLOT(playFromPoint(long)));
     connect(drawSoundWidget,SIGNAL(playFromTo(long,long)),this,SLOT(playFromTo(long,long)));
     connect(drawSoundWidget,SIGNAL(stopPlay()),this,SLOT(stopPlay()));
+    
+    #ifdef HAS_PORTMIDI
+        connect(drawSoundWidget,SIGNAL(midiAssChanged(QValueList <beatline_data>)),
+            midiInput, SLOT(setBeatlineData(QValueList <beatline_data>)));
+    #endif
 
 
     spectrumView=new SpectrumView(small_waveAndSpectrum);
@@ -416,6 +446,9 @@ Freecycle::Freecycle()
     soundPlayer->addSoundMonitor(drawSoundWidget); //Why QThread isn't QObject ?!?
     soundPlayer->addSoundMonitor(spectrumView);        //Why QThread isn't QObject ?!?
     soundPlayer->addSoundMonitor(scopeView);        //Why QThread isn't QObject ?!?
+    #ifdef HAS_PORTMIDI
+        midiInput->setSoundPlayer(soundPlayer);
+    #endif
 
 
     centralLayout->hide();
@@ -457,23 +490,27 @@ Freecycle::Freecycle()
     menuBar()->setItemEnabled(4,FALSE);
 
     QPopupMenu* helpPopupMenu=new QPopupMenu( this );
+    helpPopupMenu->insertItem( tr("Help"),this,SLOT(help()));
     helpPopupMenu->insertItem( tr("About"),this,SLOT(about()));
     menuBar()->insertItem( tr("&Help"), helpPopupMenu,5);
 
 
     openFileDialog=new MyOpenFileDialog(soundManager,soundPlayer,fileManager);
     saveFileDialog=new MySaveFileDialog();
-    projectManager=new ProjectManager(soundManager,
-                                                                             envelopeControllerWidget->getEnvData());
+    projectManager=new ProjectManager(soundManager,envelopeControllerWidget->getEnvData());
     projectManager->setLoopMarkers(drawSoundWidget->getLeftLoopMarker(),
                                                                     drawSoundWidget->getRightLoopMarker());
 
-    connect(projectManager,SIGNAL(loadFile(const QString& )), this, SLOT(load(const QString& )));
+    connect(projectManager,SIGNAL(loadFile(const QString&)), this, SLOT(load(const QString&)));
     connect(projectManager,SIGNAL(envelopeDataChanged()),
                 envelopeControllerWidget, SLOT(envDataChanged()));
     connect(projectManager,SIGNAL(addLadspa(int)),this,SLOT(addEffectFrame(int)));
-    connect(projectManager,SIGNAL(addLockedBeatlines(QValueList<long>)),
-            drawSoundWidget,SLOT(setLockedBeatLines(QValueList <long>)));
+    connect(projectManager,SIGNAL(addLockedBeatlines(QValueList<beatline_data>)),
+            drawSoundWidget,SLOT(initLockedBeatLines(QValueList <beatline_data>)));
+    connect(projectManager,SIGNAL(addStackElementS(long,long,QString,long,int,bool )),
+            drawSoundWidget,SLOT(addStackElement(long,long,QString,long,int,bool )));
+    connect(projectManager,SIGNAL(bpmModifierChangeS(QString )),
+            bpmModifDropDown,SLOT(setStringModifier(QString)));
 
     saveProjectDialog=new MySaveProjectDialog(projectManager);
     loadProjectDialog=new MyLoadProjectDialog(projectManager);
@@ -494,16 +531,25 @@ Freecycle::Freecycle()
 
     //midiOutputThread=new MidiOutputThread();
     //midiOutputThread->start();
+    
+    
+    
+    #ifdef HAS_PORTMIDI
+        midiInputThread=new MidiInputThread(midiInput);
+        midiInputThread->start();
+    #endif
 
-    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< long > )),
-        soundManager,SLOT(beatLinesChanged(QValueList< long > )));
-    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< long > )),
-        soundManager,SLOT(lockedBeatLinesChanged(QValueList< long > )));
+    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< beatline_data > )),
+        soundManager,SLOT(beatLinesChanged(QValueList< beatline_data > )));
+    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< beatline_data > )),
+        soundManager,SLOT(lockedBeatLinesChanged(QValueList< beatline_data > )));
 
     filePopupMenu->setItemEnabled(idsavedialog,FALSE);
     filePopupMenu->setItemEnabled(idsaveasdialog,FALSE);
     filePopupMenu->setItemEnabled(idsavechunksdialog,FALSE);
+    filePopupMenu->setItemEnabled(idsaveprojectasdialog,FALSE);
     filePopupMenu->setItemEnabled(idsaveprojectdialog,FALSE);
+    filePopupMenu->setItemEnabled(idsaveloopmidifile,FALSE);
 }
 
 void Freecycle::changeZoom(int value){
@@ -562,51 +608,18 @@ void Freecycle::choose()
 }
 
 
-void Freecycle::load( const QString &fileName )
+void Freecycle::load( const QString &fileName)
 {
+        drawSoundWidget->unlockBeatLines(true);
         QFile f( fileName );
         if ( !f.open( IO_ReadOnly ) )	return;
         setCaption(release+QString(" - ")+fileName);
 
-        removeEffect();
-        fileManager->setFilename(fileName);
-        soundManager->loadFile();
-        soundManager->setHasEnvelope(FALSE);
-        soundManager->setHasEnvelope(TRUE);
-        soundPlayer->initLoop();
-        soundPlayer->setPosition(0);
-
-        zoomSlider->setValue(1);
-
-        long zoomRatioDSW=(long)(soundManager->getFrames()/
-                                                drawSoundWidget->visibleWidth());
-        long zoomRatioDASW=(long)(soundManager->getFrames()/
-                                                drawAllSoundWidget->visibleWidth());
-
-        drawSoundWidget->setZoomRatio(zoomRatioDSW);
-        drawAllSoundWidget->setZoomRatio(zoomRatioDASW);
-
-        drawSoundWidget->fileLoaded();
-        drawAllSoundWidget->fileLoaded();
-
-        statusBar()->message( tr("Loaded document %1").arg(fileName), 2000 );
-
-
-        samplerateLabel->setText(tr("samplerate: %1 Hz").arg(soundManager->getRate()));
-        nbframesLabel->setText(tr("frames: %1").arg(soundManager->getFrames()));
-        channelsLabel->setText(tr("channels: %1").arg(soundManager->getChannels()));
-        zoomRatioLabel->setText(tr("zoom: %1 - %2").arg(zoomRatioDSW).arg(1));
-        drawSoundWidget->unlockBeatLines();
-
-
-
-
-
-
-        zoomSlider->setMaxValue(soundManager->getFrames()/2);
-
-
-
+        if(effectFrame) effectFrame->reinit();
+        if(!topBox->isVisible()) topBox->show();
+        if(!playAndBeatLinesControls->isVisible()) playAndBeatLinesControls->show();
+        if(!zoomBox->isVisible()) zoomBox->show();
+        if(!centralLayout->isVisible()) centralLayout->show();
         menuBar()->setItemEnabled(1,TRUE);
         menuBar()->setItemEnabled(2,TRUE);
         menuBar()->setItemEnabled(3,TRUE);
@@ -616,15 +629,40 @@ void Freecycle::load( const QString &fileName )
         filePopupMenu->setItemEnabled(idsavedialog,TRUE);
         filePopupMenu->setItemEnabled(idsaveasdialog,TRUE);
         filePopupMenu->setItemEnabled(idsavechunksdialog,TRUE);
+        filePopupMenu->setItemEnabled(idsaveprojectasdialog,TRUE);
         filePopupMenu->setItemEnabled(idsaveprojectdialog,TRUE);
-
-        if(effectFrame) effectFrame->reinit();
-        if(!topBox->isVisible()) topBox->show();
-        if(!playAndBeatLinesControls->isVisible()) playAndBeatLinesControls->show();
-        if(!zoomBox->isVisible()) zoomBox->show();
-        if(!centralLayout->isVisible()) centralLayout->show();
+        filePopupMenu->setItemEnabled(idsaveloopmidifile,TRUE);
         soundPlayer->setPosition(0);
-        soundManager->envelopeChanged(envelopeControllerWidget->getEnvData());
+
+        if(soundManager->getHasLADSPA()) removeEffect();
+        fileManager->setFilename(fileName);
+        soundManager->loadFile();
+        zoomSlider->setMaxValue(soundManager->getFrames()/2);
+        soundPlayer->initLoop();
+        soundPlayer->setPosition(0);
+        
+        
+
+        //zoomSlider->setValue(1);
+
+        long zoomRatioDSW=(long)(soundManager->getFrames()/
+                                                drawSoundWidget->visibleWidth());
+        long zoomRatioDASW=(long)(soundManager->getFrames()/
+                                                drawAllSoundWidget->visibleWidth());
+
+        drawAllSoundWidget->setZoomRatio(zoomRatioDASW,!loadingProject);
+        drawSoundWidget->putZoomRatio(zoomRatioDSW);
+        
+        if(!loadingProject){
+            drawSoundWidget->fileLoaded();
+            drawAllSoundWidget->fileLoaded();
+        }
+
+        statusBar()->message( tr("Loaded document %1").arg(fileName), 2000 );
+        samplerateLabel->setText(tr("samplerate: %1 Hz").arg(soundManager->getRate()));
+        nbframesLabel->setText(tr("frames: %1").arg(soundManager->getFrames()));
+        channelsLabel->setText(tr("channels: %1").arg(soundManager->getChannels()));
+        zoomRatioLabel->setText(tr("zoom: %1 - %2").arg(zoomRatioDSW).arg(1));
 }
 
 void Freecycle::playFromPoint(long x){
@@ -637,7 +675,6 @@ void Freecycle::playFromTo(long x1,long x2){
     soundPlayer->setOneTimeLoop(x1,x2);
     soundPlayer->setPlayingOnce(TRUE);
     play();
-    soundPlayer->setPosition(x1);
 }
 
 void Freecycle::play(){
@@ -701,80 +738,68 @@ void Freecycle::saveAs()
 
 void Freecycle::saveChunks()
 {
-    int size=drawSoundWidget->getNbBeats();
+    int size=drawSoundWidget->getNbLockedBeats();
     if(size==0){
-        QMessageBox::critical(this,"Problem!","No beatlines, no chunks! (:P)",0);
+        QMessageBox::critical(this,"Problem!","No locked beatlines, no chunks! (:P)",0);
     }else{
         QString fn=NULL;
         saveFileDialog->setBps(soundManager->getBps());
         if ( saveFileDialog->exec() == QDialog::Accepted )
             fn=saveFileDialog->selectedFile();
         if ( !fn.isEmpty() ) {
-            QMemArray <long>beatlines(0);
-            long* temp=new long[size];
-            drawSoundWidget->getBeats(temp);
-            std::sort(&temp[0],&temp[size-1]);
+            long start=0;
+            long end=soundManager->getFrames();
+            QValueList <beatline_data> beatlinesVL;
             if(saveFileDialog->exportMode()==MySaveFileDialog::EXPORT_LOOP){
                 if(drawSoundWidget->hasLoopSet()){
                     long left=drawSoundWidget->getLeftLoopMarkerSample();
                     long right=drawSoundWidget->getRightLoopMarkerSample();
-                    for (int i=0;i<size;i++){
-                        if(temp[i]>=left && temp[i]<=right){
-			/*
-                            if(beatlines.size()==0 && temp[i]!=left){ //include left marker if not present
-                                beatlines.resize(beatlines.size()+1);
-                                beatlines[beatlines.size()-1]=left;
-				cout<<"included: "<<left<<endl;
-                            }
-			    */
-                            beatlines.resize(beatlines.size()+1);
-                            beatlines[beatlines.size()-1]=temp[i];
-			    cout<<"included: "<<temp[i]<<endl;
-                        }
-                    }
-		    /*
-                    if(beatlines[beatlines.size()-1]!=right){ //include right marker if not present
-                        beatlines.resize(beatlines.size()+1);
-                        beatlines[beatlines.size()-1]=right;
-			cout<<"included: "<<right<<endl;
-                    }
-		    */
+                    beatlinesVL=drawSoundWidget->selectLockedBeatlinesList(left,right);
+                    start=left;
+                    end=right;
                 }else{
                     QMessageBox::critical(this,"Problem!","Invalid loop!",0);
                     return;
                 }
             }else{
-                beatlines.assign(temp,size);
-                std::sort(&beatlines[0],&beatlines[size-1]);
-                if(beatlines[0]!=0){ //include first frame if not present
-                    beatlines.resize(beatlines.size()+1);
-                    beatlines[beatlines.size()-1]=0;
-                    std::sort(&beatlines[0],&beatlines[size-1]);
-                }
-                if(beatlines[size-1]!=soundManager->getFrames()-1){ //include last frame if not present
-                    beatlines.resize(beatlines.size()+1);
-                    beatlines[beatlines.size()-1]=soundManager->getFrames()-1;
-                    std::sort(&beatlines[0],&beatlines[size-1]);
-                }
+                beatlinesVL=drawSoundWidget->getLockedBeatList();
             }
             cout<<"Export bits per sample: "<<saveFileDialog->getExportBitsPerSample()<<" code: "<<soundManager->toBpsCode(saveFileDialog->getExportBitsPerSample())<<"\n";
             /*
             if(saveFileDialog->getAttackTime()!=0){
                 Envelope* env=new Envelope();
-                env->setEnvDurationAndSamplerate(saveFileDialog->getAttackTime(),soundManager->getRate());
+                env->setEnvDurationAndSamplerate(saveFileDialog->getAttackTime(),
+                                                 soundManager->getRate());
                 soundManager->addPostProcessEffect(env);
             }
             */
+            
             QStringList* filenames=
-                soundManager->saveChunksTo(fn,beatlines.data(),beatlines.size(),
-                                     soundManager->toBpsCode(saveFileDialog->getExportBitsPerSample()));
+                soundManager->saveChunksTo(fn,beatlinesVL,
+                              soundManager->toBpsCode(saveFileDialog->getExportBitsPerSample()));
+            if(saveFileDialog->assignMissingMIDINotes()){
+                beatlinesVL=drawSoundWidget->assignMissingMidi(beatlinesVL);
+            }
             if(saveFileDialog->exportMidifile()){
-                midiManager->writeMidiFile(fn,beatlines.data(),
-                                                                    beatlines.size(),drawSoundWidget->getUsedBPM());
+                midiManager->writeMidiFile(fn,beatlinesVL,drawSoundWidget->getUsedBPM());
             }
             if(saveFileDialog->exportAKP()){
-                akpManager->writeAKPFile(fn,filenames);
+                akpManager->writeAKPFile(fn,filenames,beatlinesVL);
             }
+            #ifdef HAS_LIBINSTPATCH
+                if(saveFileDialog->exportSF2()){
+                    sf2Manager->writeSF2File(fn,filenames,beatlinesVL);
+                }
+                if(saveFileDialog->exportDLS()){
+                    dlsManager->writeDLS2File(fn,filenames,beatlinesVL);
+                }
+                if(saveFileDialog->exportGiga()){
+                    gigaManager->writeGigaFile(fn,filenames,beatlinesVL);
+                }
+                if(saveFileDialog->removeChunks()){
+                    fileManager->deleteDir(fn);
+                }
+            #endif
             statusBar()->message( tr( "File %1 saved to chunks" ).arg( fn), 2000 );
             delete filenames;
         } else {
@@ -791,6 +816,16 @@ void Freecycle::closeEvent( QCloseEvent* ce )
         if(soundPlayer->running()){
             soundPlayer->exit(0);
         }
+        #ifdef HAS_PORTMIDI
+            QProgressDialog pd("Terminating midi thread",0,2,this,"progress",TRUE);
+            cout<<"Terminating midi thread\n";
+            pd.setMinimumDuration(0);
+            pd.setProgress(0);
+            mainApplication->processEvents();
+            delete midiInputThread;
+            pd.setProgress(1);
+            cout<<"..ok\n";
+        #endif
         delete soundPlayer;
         delete fileManager;
         ce->accept();
@@ -800,13 +835,13 @@ void Freecycle::closeEvent( QCloseEvent* ce )
 
 void Freecycle::about()
 {
-    QMessageBox::information( this, tr("Freecycle alpha 0.1"),
+    QMessageBox::information( this, tr(release),
 			tr("Developped under GNU Public Licence by:"
                         "\nPredrag Viceic <viceic@net2000.ch>"
                         "\n\nCredits:\n\n"
                         "JACK layer  and LADSPA control port init taken from the AlsaModularSynth project by Matthias Nagorni"
                             "\nhttp://alsamodular.sourceforge.net/\n\n"
-                        "/dev/dsp layer borrowed from the libsndfile examples by Erik de Castro Lopo\n"
+                        "OSS layer borrowed from the libsndfile examples by Erik de Castro Lopo\n"
                             "http://www.mega-nerd.com/libsndfile/\n\n"
                         "Midi file generation code stolen from the Waon project\n"
                             "Copyright (C) 1998 Kengo ICHIKI (ichiki@geocities.com)\n\n"
@@ -909,29 +944,33 @@ void Freecycle::configureAmpDomain(){
 
 void Freecycle::removeEffect(){
     soundManager->setHasLadspa(false);
-    if(soundManager->getHasEnvelope())
-        soundManager->applyEnvelope();
-    else soundManager->zapInstantEBuffer();
     if(effectFrame){
-        effectFrame->close(TRUE);
-        effectFrame=0;
+            effectFrame->close(TRUE);
+            effectFrame=0;
+        }
+    if(!loadingProject){
+        if(soundManager->getHasEnvelope())
+            soundManager->applyEnvelope();
+        else soundManager->zapInstantEBuffer();
+        
+        drawSoundWidget->zapWaveCanvasBuffers();
+        drawAllSoundWidget->zapWaveCanvasBuffers();
     }
-    drawSoundWidget->zapWaveCanvasBuffers();
-    drawAllSoundWidget->zapWaveCanvasBuffers();
     projectManager->setLadspa(0);
     soundManager->getEffects()->setCurrent(0);
+    
 }
 
 void Freecycle::addEffectFrame(int uniqueId){
-
-    removeEffect();
-    soundManager->setHasLadspa(TRUE);
+    if(soundManager->getHasLADSPA()) removeEffect();
     LADSPAEffect* temp=soundManager->getEffects()->getEffect(uniqueId);
     soundManager->getEffects()->setCurrent(temp);
+    soundManager->setHasLadspa(TRUE);
     effectFrame=new EffectFrame(temp,big_and_small_wave_and_effects);
     big_and_small_wave_and_effects->setResizeMode(effectFrame,QSplitter::Stretch);
     effectFrame->setZoomRatio(drawSoundWidget->getZoomRatio());
     effectFrame->setContentsPos(drawSoundWidget->contentsX(),0);
+    effectFrame->show();
     connect(effectFrame,SIGNAL(applyMe(LADSPAEffect*)),
                     soundManager,SLOT(applyPlugin(LADSPAEffect* )));
     connect(effectFrame,SIGNAL(applyMe(LADSPAEffect*,long,long)),
@@ -943,16 +982,15 @@ void Freecycle::addEffectFrame(int uniqueId){
                     effectFrame,SLOT(setContentsPos(int,int)));
     connect(soundManager,SIGNAL(reinitLADSPAValues()),effectFrame,SLOT(reinitLADSPAValues()));
     /*
-    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< long > )),
-            effectFrame,SIGNAL(beatLinesChanged(QValueList< long > )));
+    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< beatline_data > )),
+            effectFrame,SIGNAL(beatLinesChanged(QValueList< beatline_data > )));
     */
-    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< long > )),
-            effectFrame,SLOT(lockedBeatLinesChanged(QValueList< long > )));
+    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< beatline_data > )),
+            effectFrame,SLOT(lockedBeatLinesChanged(QValueList< beatline_data > )));
 
     connect(soundManager,SIGNAL(cropped(long, long )),effectFrame,SIGNAL(cropS(long, long )));
     effectFrame->lockedBeatLinesChanged(drawSoundWidget->getLockedBeatList());
-
-    effectFrame->show();
+    
 
     //load polylines and splines, if any
     if(projectManager->getPolylines().count()>0){
@@ -974,8 +1012,7 @@ void Freecycle::addEffectFrame(int uniqueId){
     effectFrame->updateDrawModeButton();
     //end load polylines and splines, if any
 
-
-    effectFrame->apply();
+    soundManager->applyPlugin(temp);
     projectManager->setLadspa(effectFrame);
 }
 
@@ -1008,8 +1045,9 @@ void Freecycle::setFrequencyDomain(){
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
 #endif
 }
-#ifdef HAS_AUBIO
+
 void Freecycle::setComplexDomain(){
+     #ifdef HAS_AUBIO
     std::cout<<"set complex"<<endl;
     soundManager->setAnalysisMethod(SoundManager::COMPLEX_DOMAIN);
     drawSoundWidget->reinitCanvas();
@@ -1019,8 +1057,10 @@ void Freecycle::setComplexDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,FALSE);
     configureAnalysisPopupMenu->setItemChecked(5,FALSE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    #endif
 }
 void Freecycle::setPhaseDomain(){
+     #ifdef HAS_AUBIO
     std::cout<<"set phase"<<endl;
     soundManager->setAnalysisMethod(SoundManager::PHASE_DIFF);
     drawSoundWidget->reinitCanvas();
@@ -1030,8 +1070,10 @@ void Freecycle::setPhaseDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,TRUE);
     configureAnalysisPopupMenu->setItemChecked(5,FALSE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    #endif
 }
 void Freecycle::setSpectralDomain(){
+     #ifdef HAS_AUBIO
     std::cout<<"set spectral"<<endl;
     soundManager->setAnalysisMethod(SoundManager::SPEC_DIFF);
     drawSoundWidget->reinitCanvas();
@@ -1041,8 +1083,10 @@ void Freecycle::setSpectralDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,FALSE);
     configureAnalysisPopupMenu->setItemChecked(5,TRUE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    #endif
 }
 void Freecycle::setHFCDomain(){
+         #ifdef HAS_AUBIO
         std::cout<<"set hfc"<<endl;
         soundManager->setAnalysisMethod(SoundManager::HFC);
         drawSoundWidget->reinitCanvas();
@@ -1052,8 +1096,9 @@ void Freecycle::setHFCDomain(){
         configureAnalysisPopupMenu->setItemChecked(4,FALSE);
         configureAnalysisPopupMenu->setItemChecked(5,FALSE);
         configureAnalysisPopupMenu->setItemChecked(6,TRUE);
+        #endif
 }
-#endif
+
 
 void Freecycle::setDerivateSignal(){
 #ifdef HAS_AUBIO
@@ -1196,16 +1241,17 @@ void Freecycle::numPad1Pressed()
 
 
 /*!
-    \fn Freecycle::saveProject()
+    \fn Freecycle::saveProjectAs()
  */
-void Freecycle::saveProject()
+void Freecycle::saveProjectAs()
 {
     QString fn=NULL;
     if ( saveProjectDialog->exec() == QDialog::Accepted ){
         fn=saveProjectDialog->selectedFile();
         projectManager->setFilename(fn);
-        projectManager->setLockedBeatLines(
-                drawSoundWidget->getLockedBeatList());
+        projectManager->setLockedBeatLines(drawSoundWidget->getLockedBeatList());
+        projectManager->setLockedBeatRects(drawSoundWidget->getLockedBeatRects());
+        projectManager->setBpmModifier(bpmModifDropDown->getStringModifier());
         projectManager->saveProject();
     }
 }
@@ -1219,8 +1265,96 @@ void Freecycle::loadProject()
     QString fn=NULL;
     if ( loadProjectDialog->exec() == QDialog::Accepted ){
             fn=loadProjectDialog->selectedFile();
-            projectManager->loadProject(fn);
-            drawSoundWidget->analysisFinished(); //force bpm computation based on loaded beatlines
+            loadProject(fn);
     }
 
 }
+
+
+/*!
+    \fn Freecycle::configureMIDI()
+ */
+void Freecycle::configureMIDI()
+{
+#ifdef HAS_PORTMIDI
+        midiConfigDialog->exec();
+#endif
+}
+
+
+/*!
+    \fn Freecycle::saveLoopMidiFile()
+ */
+void Freecycle::saveLoopMidiFile()
+{
+    QString filename="export.mid";
+
+    QString fn = QFileDialog::getSaveFileName( filename, "MIDI files ( *.mid )",
+					       this );
+
+    if ( !fn.isEmpty() ) {
+        QValueList <beatline_data> beatlinesVL;
+        if(drawSoundWidget->hasLoopSet()){
+            long left=drawSoundWidget->getLeftLoopMarkerSample();
+            long right=drawSoundWidget->getRightLoopMarkerSample();
+            beatlinesVL=drawSoundWidget->selectLockedBeatlinesList(left,right);
+        }else{
+            QMessageBox::critical(this,"Problem!","Invalid loop!",0);
+            return;
+        }
+    
+        midiManager->writeMidiFile(fn,beatlinesVL,drawSoundWidget->getUsedBPM());
+        statusBar()->message( tr( "Midi File  %1 saved" ).arg( fn), 2000 );
+    } else {
+	statusBar()->message( tr("Saving aborted"), 2000 );
+    }
+}
+
+
+/*!
+    \fn Freecycle::saveProject()
+ */
+void Freecycle::saveProject()
+{
+    if(projectManager->getFilename()!=""){
+        projectManager->setLockedBeatLines(drawSoundWidget->getLockedBeatList());
+        projectManager->setLockedBeatRects(drawSoundWidget->getLockedBeatRects());
+        projectManager->setBpmModifier(bpmModifDropDown->getStringModifier());
+        projectManager->saveProject();
+    }else saveProjectAs();
+}
+
+
+/*!
+    \fn Freecycle::loadProject(QString fn)
+ */
+void Freecycle::loadProject(QString fn)
+{
+    if(fn!=NULL){
+        soundManager->setLoadingProject(TRUE);
+        loadingProject=TRUE;
+        projectManager->loadProject(fn);
+        loadingProject=FALSE;
+        soundManager->setLoadingProject(FALSE);
+        if(!soundManager->getHasLADSPA()) soundManager->applyEnvelope();
+        setCaption(release+QString(" - ")+fn);
+        drawSoundWidget->fileLoaded();
+    }
+}
+
+
+/*!
+    \fn Freecycle::help()
+ */
+void Freecycle::help()
+{
+    helpManager->open();
+}
+
+
+void Freecycle::setMainApplication(QApplication* theValue) {
+    mainApplication = theValue;
+    drawSoundWidget->setMainApplication(mainApplication);
+    soundPlayer->setMainApplication(mainApplication);
+    
+}
