@@ -48,7 +48,11 @@
 #include "effectframe.h"
 #include "midimanager.h"
 #include "akpmanager.h"
-#include "sf2manager.h"
+#ifdef HAS_LIBINSTPATCH
+    #include "sf2manager.h"
+    #include "gigamanager.h"
+    #include "dlsmanager.h"
+#endif
 #include "mysavefiledialog.h"
 #include "myopenfiledialog.h"
 #include "spectrumview.h"
@@ -61,10 +65,20 @@
 #include "volumeslider.h"
 #include "mysplitter.h"
 #include "envelopecontrollerwidget.h"
-#include "envelopeholder.h"
+#include "replacedwaveholder.h"
 #include "projectmanager.h"
 #include "mysaveprojectdialog.h"
 #include "myloadprojectdialog.h"
+#ifdef HAS_PORTMIDI
+    #include "midiinput.h"
+    #include "midiinputthread.h"
+    #include "midiconfigdialog.h"
+#endif
+#include "sceneframe.h"
+#include "helpmanager.h" 
+#include <qapplication.h>
+
+
 
 class QTextEdit;
 
@@ -79,13 +93,20 @@ public:
 protected:
     void closeEvent( QCloseEvent* );
 public slots:
-    void load( const QString &fileName );
+    void load( const QString &fileName);
+    void loadProject(QString fn);
+    void saveProject();
+    void help();
+    void configureMIDI();
+    void setMainApplication(QApplication* theValue);
+	
+
 private slots:
     void choose();
     void save();
     void saveAs();
     void saveChunks();
-    void saveProject();
+    void saveProjectAs();
     void changeZoom(int value);
     void changeZoom();
     void changeTreshold(int value);
@@ -129,6 +150,7 @@ private slots:
     void numPad9Pressed();
     void numPad1Pressed();
     void loadProject();
+    void saveLoopMidiFile();
 private:
     DrawSoundWidget* drawSoundWidget;
     DrawAllSoundWidget* drawAllSoundWidget;
@@ -149,7 +171,9 @@ private:
     MyOpenFileDialog* openFileDialog;
     MySaveFileDialog* saveFileDialog;
     EnvelopeControllerWidget* envelopeControllerWidget;
-    EnvelopeHolder* envelopeHolder;
+    ReplacedWaveHolder* replacedWaveHolder;
+    SceneFrame* sceneFrame;
+    HelpManager* helpManager;
 
 protected:
     QHBox* playButtons;
@@ -159,8 +183,10 @@ protected:
     QPopupMenu* configureAnalysisPopupMenu;
     QPopupMenu* configureViewPopupMenu;
     SoundPlayer* soundPlayer;
-    FloatSpinBox* newBPM;
-    QPushButton* resampleButton;
+    #ifdef HAS_SOUNDTOUCH
+        FloatSpinBox* newBPM;
+        QPushButton* resampleButton;
+    #endif
     SoundManager* soundManager;
     QPushButton* zoomToSelButton;
     QHBox* zoomBox;
@@ -182,12 +208,26 @@ protected:
     EffectFrame* effectFrame;
     MidiManager* midiManager;
     AKPManager* akpManager;
+#ifdef HAS_LIBINSTPATCH
+    SF2Manager* sf2Manager;
+    GigaManager* gigaManager;
+    DLSManager* dlsManager;
+#endif
     MidiOutputThread* midiOutputThread;
     QPopupMenu* filePopupMenu;
-    int idsavedialog, idsaveasdialog,idsavechunksdialog, idsaveprojectdialog,idloadprojectdialog;
+    int idsavedialog, idsaveasdialog,idsavechunksdialog,
+        idsaveprojectasdialog, idsaveprojectdialog, idloadprojectdialog, idsaveloopmidifile;
     ProjectManager* projectManager;
     MySaveProjectDialog* saveProjectDialog;
     MyLoadProjectDialog* loadProjectDialog;
+    bool loadingProject;
+    QApplication* mainApplication;
+    #ifdef HAS_PORTMIDI
+        MidiInput* midiInput;
+        MidiInputThread* midiInputThread;
+        MidiConfigDialog* midiConfigDialog;
+    #endif
+    
 };
 
 
