@@ -105,12 +105,12 @@ public:
         }
         int myround(){
             summedVal+=val;
-            roundedVal+=round(val);
+            roundedVal+=int(round(val));
             summederror+=roundedVal-summedVal;
             if(fabs(summederror)>1){
                 localerror=int(summederror);
                 summederror-=localerror;
-                roundedVal-=localerror;
+                roundedVal-=int(localerror);
             }
             return roundedVal;
         }
