@@ -63,6 +63,7 @@ protected:
     bool redraw;
     int oldPositionLineX;
     bool moving;
+    bool fileScrewed;
     void resizeEvent ( QResizeEvent * ) ;
     void mousePressEvent ( QMouseEvent * e );
     void mouseMoveEvent ( QMouseEvent * e );
@@ -74,6 +75,9 @@ public slots:
     void setDisplayedLength(int mdl);
     void setStartDisplayPosition(int sdp);
 
+	long getPlayingPosition() const;
+	
+
 private:
     QColor leftBgColor;
     QColor rightBgColor;
