@@ -87,11 +87,12 @@ sf_count_t SoundHolder::getFrames()
  */
 float* SoundHolder::exportLeftBuffer(long start_sample,long end_sample,bufferType type)
 {
+   float* leftAudio=getLeftChannel(type);
    assert(start_sample>=0 && end_sample<=getFrames());
    float* buffer=new float[(end_sample-start_sample)];
    int count=0;
    for (long i=start_sample;i<end_sample;i++){
-       buffer[count++]=getLeftChannel(type)[i];
+       buffer[count++]=leftAudio[i];
    }
    return buffer;
 }
@@ -101,11 +102,12 @@ float* SoundHolder::exportLeftBuffer(long start_sample,long end_sample,bufferTyp
  */
 float* SoundHolder::exportRightBuffer(long start_sample,long end_sample,bufferType type)
 {
+   float* rightAudio=getRightChannel(type);
    assert(start_sample>=0 && end_sample<=getFrames());
    float* buffer=new float[(end_sample-start_sample)];
    int count=0;
    for (long i=start_sample;i<end_sample;i++){
-       buffer[count++]=getRightChannel(type)[i];
+       buffer[count++]=rightAudio[i];
    }
    return buffer;
 }
@@ -116,14 +118,15 @@ float* SoundHolder::exportRightBuffer(long start_sample,long end_sample,bufferTy
 float* SoundHolder::exportDerivatedLeftBuffer(long start_sample,
                                               long end_sample,bufferType type)
 {
+   float* leftAudio=getLeftChannel(type);
    assert(start_sample>=0 && end_sample<=getFrames());
    float* buffer=new float[end_sample-start_sample];
    int count=0;
    for (long i=start_sample;i<end_sample-1;i++){
-       if(i==0) buffer[count++]=getLeftChannel(type)[i+1]-getLeftChannel(type)[i];
+       if(i==0) buffer[count++]=leftAudio[i+1]-leftAudio[i];
        else if(i==getFrames()-1)
-           buffer[count++]=getLeftChannel(type)[i]-getLeftChannel(type)[i-1];
-       else buffer[count++]=getLeftChannel(type)[i+1]-getLeftChannel(type)[i-1];
+           buffer[count++]=leftAudio[i]-leftAudio[i-1];
+       else buffer[count++]=leftAudio[i+1]-leftAudio[i-1];
    }
    return buffer;
 }
@@ -134,14 +137,15 @@ float* SoundHolder::exportDerivatedLeftBuffer(long start_sample,
 float* SoundHolder::exportDerivatedRightBuffer(long start_sample,
                                                long end_sample,bufferType type)
 {
+    float* rightAudio=getRightChannel(type);
    assert(start_sample>=0 && end_sample<=getFrames());
    float* buffer=new float[end_sample-start_sample];
    int count=0;
    for (long i=start_sample;i<end_sample;i++){
-       if(i==0) buffer[count++]=getRightChannel(type)[i+1]-getRightChannel(type)[i];
+       if(i==0) buffer[count++]=rightAudio[i+1]-rightAudio[i];
        else if(i==getFrames()-1)
-           buffer[count++]=getRightChannel(type)[i]-getRightChannel(type)[i-1];
-       else buffer[count++]=getRightChannel(type)[i+1]-getRightChannel(type)[i-1];
+           buffer[count++]=rightAudio[i]-rightAudio[i-1];
+       else buffer[count++]=rightAudio[i+1]-rightAudio[i-1];
    }
    return buffer;
 }
@@ -153,10 +157,11 @@ float* SoundHolder::exportDerivatedRightBuffer(long start_sample,
 void SoundHolder::importLeftBuffer(float* buffer,long start_sample,
                                    long end_sample,bufferType type)
 {
+    float* leftAudio= getLeftChannel(type);
     assert(start_sample>=0 && end_sample<=getFrames());
     int count=0;
     for (int i=start_sample;i<end_sample;i++){
-        getLeftChannel(type)[i]=buffer[count++];
+        leftAudio[i]=buffer[count++];
     }
 }
 
@@ -167,10 +172,11 @@ void SoundHolder::importLeftBuffer(float* buffer,long start_sample,
 void SoundHolder::importRightBuffer(float* buffer,long start_sample,
                                      long end_sample,bufferType type)
 {
+    float* rightAudio= getRightChannel(type);
     assert(start_sample>=0 && end_sample<=getFrames());
     int count=0;
     for (int i=start_sample;i<end_sample;i++){
-        getRightChannel(type)[i]=buffer[count++];
+        rightAudio[i]=buffer[count++];
     }
 }
 
@@ -181,9 +187,11 @@ void SoundHolder::importMultiplexedBuffer(float* buffer,long length,bufferType t
 {
     int count=0;
     initBuffer(type,length);
+    float* leftAudio=getLeftChannel(type);
+    float* rightAudio=getRightChannel(type);
     for (int i=0;i<length;i++){
-        getLeftChannel(type)[i]=buffer[count++];
-        getRightChannel(type)[i]=buffer[count++];
+        leftAudio[i]=buffer[count++];
+        rightAudio[i]=buffer[count++];
     }
     setNbFrames(length);
 }
@@ -192,9 +200,11 @@ void SoundHolder::importMultiplexedShortBuffer(short* buffer,long length,bufferT
 {
     int count=0;
     initBuffer(type,length);
+    float* leftAudio=getLeftChannel(type);
+    float* rightAudio=getRightChannel(type);
     for (int i=0;i<length;i++){
-        getLeftChannel(type)[i]=(float)(buffer[count++]/float(SHRT_MAX));
-        getRightChannel(type)[i]=(float)(buffer[count++]/float(SHRT_MAX));
+        leftAudio[i]=(float)(buffer[count++]/float(SHRT_MAX));
+       rightAudio[i]=(float)(buffer[count++]/float(SHRT_MAX));
     }
     setNbFrames(length);
 }
@@ -207,9 +217,11 @@ float* SoundHolder::exportMultiplexedBuffer(long start_sample,long end_sample,
     assert(end_sample>start_sample);
    float* buffer=new float[(end_sample-start_sample)*getChannels()];
    int count=0;
+   float* leftAudio=getLeftChannel(type);
+    float* rightAudio=getRightChannel(type);
    for (long i=start_sample;i<end_sample;i++){
-       buffer[count++]=getLeftChannel(type)[i];
-       buffer[count++]=getRightChannel(type)[i];
+       buffer[count++]=leftAudio[i];
+       buffer[count++]=rightAudio[i];
    }
    return buffer;
 }
@@ -221,22 +233,25 @@ float* SoundHolder::exportMultiplexedBuffer(long start_sample,long end_sample,
 void SoundHolder::exportMultiplexedBuffer(float* buffer, long start_sample,
                 long end_sample,bufferType type)
 {
-   assert(start_sample>=0 && end_sample<=getFrames());
+   float* leftAudio=getLeftChannel(type);
+    float* rightAudio=getRightChannel(type);
    int count=0;
    for (long i=start_sample;i<end_sample;i++){
-       buffer[count++]=(getLeftChannel(type)[i]);
-       buffer[count++]=(getRightChannel(type)[i]);
+       buffer[count++]=(leftAudio[i]);
+       buffer[count++]=(rightAudio[i]);
    }
 }
 
 void SoundHolder::exportMultiplexedShortBuffer(short* buffer,
                                                long start_sample,long end_sample,bufferType type)
 {
+   float* leftAudio=getLeftChannel(type);
+    float* rightAudio=getRightChannel(type);
    assert(start_sample>=0 && end_sample<=getFrames());
    int count=0;
    for (long i=start_sample;i<end_sample;i++){
-       buffer[count++]=(short)(getLeftChannel(type)[i]*float(SHRT_MAX));
-       buffer[count++]=(short)(getRightChannel(type)[i]*float(SHRT_MAX));
+       buffer[count++]=(short)(leftAudio[i]*float(SHRT_MAX));
+       buffer[count++]=(short)(rightAudio[i]*float(SHRT_MAX));
    }
 }
 
@@ -350,6 +365,7 @@ void SoundHolder::zap(bufferType type)
  */
 void SoundHolder::initBuffer(bufferType type, long length)
 {
+    cout<<"+++++++init buffer type: "<<type<<" no: "<<getBufferIndex(type)<<" length: "<<length<<endl;
     buffers[getBufferIndex(type)].init(length,ANY);
 }
 
@@ -359,12 +375,12 @@ void SoundHolder::initBuffer(bufferType type, long length)
  */
 int SoundHolder::getBufferIndex(bufferType type)
 {
+    //if(type==FOR_OUTPUT) cout<<"type asked: "<<type<<" hasLadspa: "<<hasLADSPA<<" hasEnvelope "<<hasEnvelope<<endl;
     switch(type){
         case ORIGINAL_WAVE:
         case BEFORE_LADSPA:
         {
             return 0;
-            break;
         }
         case AFTER_LADSPA:
         case BEFORE_ENVELOPE:{
@@ -439,15 +455,17 @@ int SoundHolder::loadBuffer(QString path,bufferType type){
     sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)sfinfo.frames);
     //demux channels
     int count=0;
+    float* leftAudio= getLeftChannel(type);
+    float* rightAudio= getRightChannel(type);
     for (int i=0;i<sfinfo.frames;i++){
-        getLeftChannel(type)[i]=wavefile[count];
+        leftAudio[i]=wavefile[count];
         if(channels==2)
-            getRightChannel(type)[i]=wavefile[count+1];
+            rightAudio[i]=wavefile[count+1];
         else if(channels==1){
-            getRightChannel(type)[i]=wavefile[count];
+            rightAudio[i]=wavefile[count];
         }
         count+=sfinfo.channels;
-        if(i%100000==0){
+        if(i%10000==0){
             progress.setProgress(i);
             progress.setLabelText("Loading File...(frame: "+QString::number(i)+")");
         }
@@ -562,30 +580,75 @@ bool SoundHolder::getHasLADSPA()
 /*!
     \fn SoundHolder::insertFileAt(QString filename,long start_sample)
  */
-void SoundHolder::insertFileAt(QString filename,long start_sample)
+long SoundHolder::insertFileAt(QString filename,long start_sample)
 {
-    SNDFILE* tmp_sndfile= sf_open(filename, SFM_READ, &sfinfo);
-    if (!tmp_sndfile) return ;
+    
+    
+    SF_INFO tempsfinfo;
+    SNDFILE* tmp_sndfile= sf_open(filename, SFM_READ, &tempsfinfo);
+    if (!tmp_sndfile) return 0;
 
-    float* wavefile=new float[sfinfo.frames*2/*sfinfo.channels*/];
-    sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)sfinfo.frames);
+    float* wavefile=new float[tempsfinfo.frames*2/*sfinfo.channels*/];
+    sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)tempsfinfo.frames);
 
-    enlargeBuffersAt(start_sample,sfinfo.frames);
+    enlargeBuffersAt(start_sample,tempsfinfo.frames);
 
     //demux channels
     int count=0;
+    float* leftAudio= getLeftChannel(ORIGINAL_WAVE);
+    float* rightAudio= getRightChannel(ORIGINAL_WAVE);
     for (int i=start_sample;i<start_sample+sfinfo.frames;i++){
-            //getLeftChannel(type)[i]=wavefile[count];
+          leftAudio[i]=wavefile[count];
         if(channels==2){
-            //getRightChannel(type)[i]=wavefile[count+1];
-        }else if(channels==1){
-            //getRightChannel(type)[i]=wavefile[count];
+            rightAudio[i]=wavefile[count+1];
         }
     }
     //end demux channels
 
-    zaparr(wavefile);
     sf_close(tmp_sndfile);
+    setNbFrames(getFrames()+tempsfinfo.frames);
+    return tempsfinfo.frames;
+}
+
+
+/*!
+    \fn SoundHolder::replaceFileAt(QString filename,long start_sample,long end_sample)
+ */
+void SoundHolder::replaceFileAt(QString filename,long start_sample,long end_sample)
+{
+   
+    SF_INFO tempsfinfo;
+    SNDFILE* tmp_sndfile= sf_open(filename, SFM_READ, &tempsfinfo);
+    if (!tmp_sndfile){
+        cout<<"Error opening "<<filename<<endl;
+        return;
+    }
+
+    float* wavefile=new float[tempsfinfo.frames*2/*sfinfo.channels*/];
+    sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)tempsfinfo.frames);
+
+    //demux channels
+    int count=0;
+     float* leftAudio=getLeftChannel(ORIGINAL_WAVE);
+    float* rightAudio=getRightChannel(ORIGINAL_WAVE);
+    for (int i=start_sample;i<end_sample;i++){
+        if(i<start_sample+tempsfinfo.frames){
+            leftAudio[i]=wavefile[count++];
+            if(channels==2){
+                rightAudio[i]=wavefile[count++];
+            }
+        }else{
+            leftAudio[i]=0;
+            if(channels==2){
+                rightAudio[i]=0;
+            }
+        }
+    }
+    //end demux channels
+
+    sf_close(tmp_sndfile);
+    zaparr(wavefile);
+    cout<<"File inserted from "<<start_sample<<" to "<<end_sample<<"\n";
 }
 
 
