@@ -20,13 +20,15 @@
 #include "soundmanager.h"
 #include <stdio.h>
 
-using namespace soundtouch;
+#ifdef HAS_SOUNDTOUCH
+    using namespace soundtouch;
+#endif
 
 
 SoundManager::SoundManager():SoundHolder(),FFTHolder()
 {
         instant_buffer=0;
-
+        loadingProject=FALSE;
         amp_domain_instant_energy_buffersize=pow(2,8);
         amp_domain_average_energy_buffersize=pow(2,11);
         analysisMethod=AMP_DOMAIN;
@@ -79,6 +81,7 @@ QMemArray<long> SoundManager::analyse(const double treshold,
 QMemArray<long> SoundManager::analyse_freq_domain(const double treshold,
                                                                                    const int window,long start_sample,long end_sample)
 {
+    float* leftAudio=getLeftChannel(ORIGINAL_WAVE);
     assert(start_sample>=0 && end_sample<=getFrames());
     double max_variance=0;
     double min_variance=10;
@@ -163,7 +166,7 @@ QMemArray<long> SoundManager::analyse_freq_domain(const double treshold,
                             tempbeats[tempbeats.size()-1].band=k;
                             tempbeats[tempbeats.size()-1].instant_energy=temp_instant;
 
-                            tempbeats[tempbeats.size()-1].energy_at_pos=getLeftChannel(ORIGINAL_WAVE)[i];
+                            tempbeats[tempbeats.size()-1].energy_at_pos=leftAudio[i];
 
                             /*
                             tempbeats[tempbeats.size()-1].min_amplitude=min_amplitude;
@@ -272,16 +275,16 @@ QMemArray<long> SoundManager::analyse_freq_domain(const double treshold,
 
         switch(analysisMethod) {
             case COMPLEX_DOMAIN:
-                aubio_method = complexdomain;
+                aubio_method = aubio_onset_complex;
                 break;
             case PHASE_DIFF:
-                aubio_method = phase;
+                aubio_method = aubio_onset_phase;
                     break;
             case SPEC_DIFF:
-                aubio_method = specdiff;
+                aubio_method = aubio_onset_specdiff;
                 break;
             case HFC:
-                aubio_method = hfc;
+                aubio_method = aubio_onset_hfc;
                     break;
             default:
                 break;
@@ -351,6 +354,7 @@ QMemArray<long> SoundManager::analyse_freq_domain(const double treshold,
 QMemArray<long> SoundManager::analyse_time_domain(const double treshold,
                                                                                    const int window,long start_sample,long end_sample)
 {
+        float* leftAudio=getLeftChannel(ORIGINAL_WAVE);
         //double max_variance=30000;
         //double min_variance=30000;
         assert(start_sample>=0 && end_sample<=getFrames());
@@ -491,18 +495,18 @@ QMemArray<long> SoundManager::analyse_time_domain(const double treshold,
                             float right_half_energy=0;
                             //find absolute minima and maxima
                             for (int count=i;count<i+amp_domain_instant_energy_buffersize;count++){
-                                if(fabs(getLeftChannel(ORIGINAL_WAVE)[count]) <=min_amplitude){
-                                    min_amplitude=fabs(getLeftChannel(ORIGINAL_WAVE)[count]);
+                                if(fabs(leftAudio[count]) <=min_amplitude){
+                                    min_amplitude=fabs(leftAudio[count]);
                                     min_pos=count;
                                 }
-                                if(fabs(getLeftChannel(ORIGINAL_WAVE)[count]) >=max_amplitude){
-                                    max_amplitude=fabs(getLeftChannel(ORIGINAL_WAVE)[count]);
+                                if(fabs(leftAudio[count]) >=max_amplitude){
+                                    max_amplitude=fabs(leftAudio[count]);
                                     max_pos=count;
                                 }
                                 if(count<i+amp_domain_instant_energy_buffersize/2){
-                                    left_half_energy+=fabs(getLeftChannel(ORIGINAL_WAVE)[count]);
+                                    left_half_energy+=fabs(leftAudio[count]);
                                 }else{
-                                    right_half_energy+=fabs(getLeftChannel(ORIGINAL_WAVE)[count]);
+                                    right_half_energy+=fabs(leftAudio[count]);
                                 }
                             }
                             //end find absolute minima and maxima
@@ -510,7 +514,7 @@ QMemArray<long> SoundManager::analyse_time_domain(const double treshold,
                             tempbeats[tempbeats.size()-1].sample_pos=pos;
                             tempbeats[tempbeats.size()-1].band=0;
                             tempbeats[tempbeats.size()-1].instant_energy=currentInstantEnergy;
-                            tempbeats[tempbeats.size()-1].energy_at_pos=getLeftChannel(ORIGINAL_WAVE)[pos];
+                            tempbeats[tempbeats.size()-1].energy_at_pos=leftAudio[pos];
                             tempbeats[tempbeats.size()-1].min_amplitude=min_amplitude;
                             tempbeats[tempbeats.size()-1].max_amplitude=max_amplitude;
                             tempbeats[tempbeats.size()-1].min_pos=min_pos;
@@ -657,6 +661,7 @@ QMemArray<long> SoundManager::analyse_time_domain(const double treshold,
         return beats;
 }
 
+#ifdef HAS_SOUNDTOUCH
 /*!
     \fn SoundManager::resample()
  */
@@ -773,6 +778,90 @@ void SoundManager::resample(list<long>sortedPos,double new_to_oldBPM_ratio)
 
 }
 
+/*!
+    \fn SoundManager::pitchshift(long start_sample, long end_sample,)
+ */
+void SoundManager::pitchshift(long start_sample, long end_sample, double ratio)
+{
+    fileManager->deleteAfterCurrent();
+        pushSoundBuffer(ORIGINAL_WAVE);
+        
+        SoundTouch* st=new SoundTouch();
+	st->setSampleRate(getRate());
+	st->setChannels(getChannels());
+	st->setTempo(1);
+	st->setPitch(ratio);
+	st->setRate(1);
+
+        long currentPos=0;
+#ifdef FLOAT_SAMPLES
+        float* outputWave=
+            new float[(int)ceil((end_sample-start_sample)*getChannels())];
+#else
+        short* outputWave=
+            new short[(int)ceil((end_sample-start_sample)*getChannels())];
+#endif
+
+
+#ifdef FLOAT_SAMPLES
+                float * tmp_wave = new float[(end_sample-start_sample)*getChannels()];
+                exportMultiplexedBuffer(tmp_wave,start_sample,end_sample,ORIGINAL_WAVE);
+                float * tmp_wave2 = new float[(end_sample-start_sample)*getChannels()];
+#else
+                short * tmp_wave = new short[(end_sample-start_sample)*getChannels()];
+                exportMultiplexedShortBuffer(tmp_wave,start_sample,end_sample,ORIGINAL_WAVE);
+                short * tmp_wave2 = new short[(end_sample-start_sample)*getChannels()];
+#endif
+
+
+		st->putSamples(tmp_wave,end_sample-start_sample);
+		uint returned=0;
+		do{
+                        returned=st->receiveSamples(tmp_wave2,(end_sample-start_sample)*getChannels());
+#ifdef FLOAT_SAMPLES
+                        outputWave=SoundHolder::insertArray(outputWave,
+                                                                    tmp_wave2,currentPos,returned);
+#else
+                        outputWave=SoundHolder::insertShortArray(outputWave,
+                                                                    tmp_wave2,currentPos,returned);
+#endif
+                       currentPos+=returned;
+		}while(returned!=0);
+                zaparr(tmp_wave);
+	st->flush();
+	returned=0;
+	do{
+		returned=st->receiveSamples(tmp_wave2,(end_sample-start_sample)*getChannels());
+#ifdef FLOAT_SAMPLES
+                outputWave=SoundHolder::insertArray(outputWave,
+                                                    tmp_wave2,currentPos,returned);
+#else
+                outputWave=SoundHolder::insertShortArray(outputWave,
+                                                    tmp_wave2,currentPos,returned);
+#endif
+
+                currentPos+=returned;
+	}while(returned!=0);
+#ifdef FLOAT_SAMPLES
+        importMultiplexedBuffer(outputWave,currentPos,ORIGINAL_WAVE);
+#else
+        importMultiplexedShortBuffer(outputWave,currentPos,ORIGINAL_WAVE);
+#endif
+
+        zaparr(tmp_wave2);
+        zaparr(outputWave);
+        if(getHasLADSPA()) initBuffer(AFTER_LADSPA,getFrames());
+        if(getHasEnvelope()) initBuffer(AFTER_ENVELOPE,getFrames());
+        getEffects()->setNbFrames(getFrames());
+        cout<<"pitchshift end\n";
+        emit(reinitLADSPAValues());
+        zapInstantEBuffer();
+        if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); //reapply current plugin
+        else if(getHasEnvelope()) applyEnvelope();
+        else emit(waveChanged());
+
+}
+#endif
 
 
 
@@ -783,6 +872,11 @@ int SoundManager::loadFile()
 {
         int result=SoundHolder::loadBuffer(fileManager->getOriginalFilename(),
                                            ORIGINAL_WAVE);
+        if(getHasEnvelope()){
+            initBuffer(AFTER_ENVELOPE,getFrames());
+            applyEnvelope(FALSE);
+        }
+        
         zapInstantEBuffer();
         return result;
 }
@@ -902,6 +996,8 @@ bool SoundManager::nextSoundBuffer(){
 }
 
 int SoundManager::loadBuffer(QString path,long start_sample,long end_sample){
+    float* leftAudio=getLeftChannel(SoundHolder::ORIGINAL_WAVE);
+    float* rightAudio=getRightChannel(SoundHolder::ORIGINAL_WAVE);
     assert(start_sample>=0 && end_sample<=getFrames());
     SNDFILE* tmp_sndfile= sf_open(path, SFM_READ, &sfinfo);
     if(tmp_sndfile==NULL) return 1;
@@ -910,9 +1006,9 @@ int SoundManager::loadBuffer(QString path,long start_sample,long end_sample){
     sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)(end_sample-start_sample));
     int count=0;
     for (int i=start_sample;i<end_sample;i++){
-        getLeftChannel(ORIGINAL_WAVE)[i]=wavefile[count];
-        getRightChannel(ORIGINAL_WAVE)[i]=wavefile[count+1];
-        count+=sfinfo.channels;
+       leftAudio[i]=wavefile[count];
+       rightAudio[i]=wavefile[count+1];
+       count+=sfinfo.channels;
     }
 
     zaparr(wavefile);
@@ -924,27 +1020,27 @@ int SoundManager::loadBuffer(QString path,long start_sample,long end_sample){
 
 
 /*!
-    \fn SoundManager::saveChunksTo(QString,long*)
+    \fn SoundManager::saveChunksTo(QString filename,QValueList <beatline_data> bld,int format)
  */
-QStringList* SoundManager::saveChunksTo(QString filename,long beats[],int length,int format)
+QStringList* SoundManager::saveChunksTo(QString filename,QValueList <beatline_data> bld,int format)
 {
     QDir dir;
     dir.mkdir(filename);
     QStringList* resultFilenames=new QStringList();
-    std::sort(beats,beats+length);
-    std::cout<<"size: "<<length<<"\n";
-    for (int i=0;i<length-1;i++){
-        QString tempfilename=QString::number(i).rightJustify(8,'0').append(".wav").prepend(filename+"/");
-        saveChunkToAs(tempfilename,beats[i],beats[i+1],format,0);
+    qHeapSort(bld);
+    for (int i=0;i<bld.count()-1;i++){
+        QString tempfilename=
+            QString::number(i).rightJustify(8,'0').append(".wav").prepend(filename+"/");
+        saveChunkToAs(tempfilename,bld[i].position,bld[i+1].position,format,0);
         resultFilenames->push_back(tempfilename.section( '/', -1 ));
     }
     postProcessEffects->clear();
     return resultFilenames;
 }
 
-QStringList* SoundManager::saveChunksTo(QString filename,long beats[],int length)
+QStringList* SoundManager::saveChunksTo(QString filename,QValueList <beatline_data> bld)
 {
-    return saveChunksTo(filename,beats,length,sfinfo.format);
+    return saveChunksTo(filename,bld,sfinfo.format);
 }
 
 
@@ -956,7 +1052,7 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
     assert(start_sample>=0 && end_sample<=getFrames());
 
     QProgressDialog progress("Applying pugin...","Abort",getFrames(),NULL,"progress",TRUE);
-    progress.setMinimumDuration(1000);
+    progress.setMinimumDuration(3000);
     progress.setProgress(0);
 
     //audio ports
@@ -984,7 +1080,7 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
                 processedSamples+=lastChunk;
                 effect->advancePortValues(lastChunk);
             }
-            if(i%(102400)==0){
+            if(progress.isVisible() && i%(102400)==0){
                 progress.setProgress(i);
                 progress.setLabelText("Applying pugin...(frame: "+QString::number(i)+")");
             }
@@ -994,6 +1090,7 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
             assert(processedSamples<=getFrames());
     }
     progress.close();
+    cout<<"Plugin applied\n";
     if(hasEnvelope){
         applyEnvelope();
     }else{
@@ -1053,6 +1150,9 @@ float** SoundManager::compute_subband_energies(int subband_number,
                                                                                                     bufferType type)
 {
 
+    float* leftAudio=getLeftChannel(type);
+    float* rightAudio=getRightChannel(type);
+    
     cout<<"Computing fft\n";
     assert(start_sample>=0 && end_sample<=getFrames());
     QProgressDialog progress("Computing FFT...","Abort",end_sample-start_sample,NULL,"progress",TRUE);
@@ -1100,7 +1200,7 @@ float** SoundManager::compute_subband_energies(int subband_number,
             //fill in and out buffers
             if(windowMode==WINDOW_GAUSS){
                 float std_dev=
-                        MyMath::standard_deviation(&getLeftChannel(type)[i],advance_frames);
+                        MyMath::standard_deviation(&leftAudio[i],advance_frames);
                 MyMath::gaussian_window(gaussian,advance_frames,MyMath::linear_regression(0,1,2,8,std_dev));
             }
             for (int i2=i;i2<i+frames;i2++){
@@ -1335,8 +1435,8 @@ int w, bool ssm)
         substractSpectralMean=ssm;
 
         //recompute_subband_energies();
-        //this->zapInstantEBuffer();
-        if(!getHasEnvelope() && !getHasLADSPA()) this->zapInstantEBuffer();
+        this->zapInstantEBuffer();
+        //if(!getHasEnvelope() && !getHasLADSPA()) this->zapInstantEBuffer();
         emit(fftParamsChanged());
         emit(analysisParamsChanged());
 }
@@ -1536,6 +1636,7 @@ QMemArray <SoundManager::beat> SoundManager::findPreciseBeat(QMemArray<beat> inp
                         int search_distance,int beatline_distance,float distance_to_min_conf,float lr_diff_conf)
 {
     QMemArray<beat> tempbeats(0); // array of beats
+    float* leftAudio=getLeftChannel(ORIGINAL_WAVE);
     for (int i=0;i<input.size();i++){
         float distance_to_min=FLT_MAX;
         float left_right_diff=0;
@@ -1554,39 +1655,39 @@ QMemArray <SoundManager::beat> SoundManager::findPreciseBeat(QMemArray<beat> inp
         for (int j=start_pos;j>end_pos;j--){
                 if(j<search_distance){
                     pos=0;
-                }else if(j+search_distance>getFrames()){
+                }else if(j+search_distance>=getFrames()){
                     pos=getFrames();
                 }else{
 		    if(search_distance==0){
-                        temp_abs_en_left=fabs(getLeftChannel(ORIGINAL_WAVE)[j]);
-                        temp_en_left+=getLeftChannel(ORIGINAL_WAVE)[j];
-                        temp_abs_en_right=fabs(getLeftChannel(ORIGINAL_WAVE)[j+1]);
-                        temp_en_right+=getLeftChannel(ORIGINAL_WAVE)[j+1];
+                        temp_abs_en_left=fabs(leftAudio[j]);
+                        temp_en_left+=leftAudio[j];
+                        temp_abs_en_right=fabs(leftAudio[j+1]);
+                        temp_en_right+=leftAudio[j+1];
 			left_energy=temp_abs_en_left;
                         right_energy=temp_abs_en_right;
 		    }else{
                         if(j==start_pos){
                             //first run, compute all
                             for (int k=j-search_distance;k<j;k++){
-                                temp_abs_en_left+=fabs(getLeftChannel(ORIGINAL_WAVE)[k]);
-                                temp_en_left+=getLeftChannel(ORIGINAL_WAVE)[k];
+                                temp_abs_en_left+=fabs(leftAudio[k]);
+                                temp_en_left+=leftAudio[k];
                             }
                             for (int k=j;k<j+search_distance;k++){
-                                temp_abs_en_right+=fabs(getLeftChannel(ORIGINAL_WAVE)[k]);
-                                temp_en_right+=getLeftChannel(ORIGINAL_WAVE)[k];
+                                temp_abs_en_right+=fabs(leftAudio[k]);
+                                temp_en_right+=leftAudio[k];
                             }
                         }else{
                             //add/substaract differences
-                            temp_abs_en_left=temp_abs_en_left-fabs(getLeftChannel(ORIGINAL_WAVE)[j])+
-                                    fabs(getLeftChannel(ORIGINAL_WAVE)[j-search_distance]);
-                            temp_en_left=temp_en_left-getLeftChannel(ORIGINAL_WAVE)[j]+
-                                    getLeftChannel(ORIGINAL_WAVE)[j-search_distance];
+                            temp_abs_en_left=temp_abs_en_left-fabs(leftAudio[j])+
+                                    fabs(leftAudio[j-search_distance]);
+                            temp_en_left=temp_en_left-leftAudio[j]+
+                                    leftAudio[j-search_distance];
 
                             temp_abs_en_right=
-                                    temp_abs_en_right-fabs(getLeftChannel(ORIGINAL_WAVE)[j+search_distance])+
-                                    fabs(getLeftChannel(ORIGINAL_WAVE)[j]);
-                            temp_en_right=temp_en_right-getLeftChannel(ORIGINAL_WAVE)[j+search_distance]+
-                                    getLeftChannel(ORIGINAL_WAVE)[j];
+                                    temp_abs_en_right-fabs(leftAudio[j+search_distance])+
+                                    fabs(leftAudio[j]);
+                            temp_en_right=temp_en_right-leftAudio[j+search_distance]+
+                                    leftAudio[j];
                         }
 		    }
                     if(temp_abs_en_left<=distance_to_min_conf*distance_to_min){
@@ -1616,7 +1717,7 @@ QMemArray <SoundManager::beat> SoundManager::findPreciseBeat(QMemArray<beat> inp
         tempbeats.resize(tempbeats.size()+1);
         tempbeats[tempbeats.size()-1]=input[i];
         tempbeats[tempbeats.size()-1].sample_pos=pos;
-        tempbeats[tempbeats.size()-1].energy_at_pos=fabs(getLeftChannel(ORIGINAL_WAVE)[pos]);
+        tempbeats[tempbeats.size()-1].energy_at_pos=fabs(leftAudio[pos]);
         tempbeats[tempbeats.size()-1].left_half_energy=left_energy;
         tempbeats[tempbeats.size()-1].right_half_energy=right_energy;
     }
@@ -1657,6 +1758,7 @@ void SoundManager::removeBeatLineAt(long pos)
  */
 void SoundManager::updateMaxAmplitudes(QMemArray<beat> beatlines, int search_distance)
 {
+    float* leftAudio=getLeftChannel(ORIGINAL_WAVE);
     for (int i=0;i<beatlines.size();i++){
         //compute the max amplitude after the beatline (for the treshold slider)
         long pos=beatlines[i].sample_pos;
@@ -1664,8 +1766,8 @@ void SoundManager::updateMaxAmplitudes(QMemArray<beat> beatlines, int search_dis
         int search_amplitude_end=pos+search_distance;
         if(search_amplitude_end>getFrames()) search_amplitude_end=getFrames();
         for (int j=pos;j<search_amplitude_end;j++){
-            if (fabs(getLeftChannel(ORIGINAL_WAVE)[j])>max_amplitude){
-                max_amplitude=fabs(getLeftChannel(ORIGINAL_WAVE)[j]);
+            if (fabs(leftAudio[j])>max_amplitude){
+                max_amplitude=fabs(leftAudio[j]);
             }
         }
         beatlines[i].max_amplitude=max_amplitude;
@@ -1692,9 +1794,10 @@ QMemArray<SoundManager::beat> SoundManager::removeRedundantBeatlines(QMemArray<b
 /*!
     \fn SoundManager::applyEnvelope()
  */
-void SoundManager::applyEnvelope()
+void SoundManager::applyEnvelope(bool forceRedraw)
 {
     if(hasEnvelope){
+        cout<<"Trying to apply envelope\n";
         float* inputBufferLeft=SoundHolder::getLeftChannel(BEFORE_ENVELOPE);
         float* inputBufferRight=SoundHolder::getRightChannel(BEFORE_ENVELOPE);
         float* outputBufferLeft=SoundHolder::getLeftChannel(AFTER_ENVELOPE);
@@ -1713,17 +1816,21 @@ void SoundManager::applyEnvelope()
             long r_duration=(envelopeData->release*envelopeData->env_duration/1000.0)*float(getRate());
             long s_duration=applyNbFrames-r_duration-ahd_duration;
             for (long j=0;j<lockedBeatLines.count();j++){
-                long end=j<lockedBeatLines.count()-1?lockedBeatLines[j+1]:getFrames();
-                for (long i=lockedBeatLines[j];i<end;i++){
+                long end=j<lockedBeatLines.count()-1?lockedBeatLines[j+1].position:getFrames();
+                for (long i=lockedBeatLines[j].position;i<end;i++){
                     if(envelopeData->env_mode!=EnvelopeData::FIT_TO_SAMPLES &&
-                    i-lockedBeatLines[j]<applyNbFrames){
-                        outputBufferLeft[i]=inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j]];
-                        outputBufferRight[i]=inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j]];
+                    i-lockedBeatLines[j].position<applyNbFrames){
+                        outputBufferLeft[i]=
+                            inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j].position];
+                        outputBufferRight[i]=
+                            inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j].position];
                     }else if(envelopeData->env_mode==EnvelopeData::FIT_TO_SAMPLES){
-                        if(i-lockedBeatLines[j]<=ahd_duration){
-                            outputBufferLeft[i]=inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j]];
-                            outputBufferRight[i]=inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j]];
-                        }else if(i-lockedBeatLines[j]>ahd_duration &&i<end-r_duration){
+                        if(i-lockedBeatLines[j].position<=ahd_duration){
+                            outputBufferLeft[i]=
+                                inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j].position];
+                            outputBufferRight[i]=
+                                inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j].position];
+                        }else if(i-lockedBeatLines[j].position>ahd_duration &&i<end-r_duration){
                             //apply sustain
                             outputBufferLeft[i]=inputBufferLeft[i]*envelopeData->sustain;
                             outputBufferRight[i]=inputBufferRight[i]*envelopeData->sustain;
@@ -1745,21 +1852,23 @@ void SoundManager::applyEnvelope()
                     }
                 }
             }
+                cout<<"Envelope applied\n";
         }
     }
-    cout<<"Envelope applied\n";
-    //force spectrogram repaint
-    if(hiresfreq_subband_energies) zaparr(hiresfreq_subband_energies);
-    //this->zapInstantEBuffer();
-    //emit(waveChanged(start_sample,end_sample));  //should be..
-    emit(waveChanged()); //is ..
+    if(forceRedraw){
+        //force spectrogram repaint
+        if(hiresfreq_subband_energies) zaparr(hiresfreq_subband_energies);
+        //this->zapInstantEBuffer();
+        //emit(waveChanged(start_sample,end_sample));  //should be..
+        emit(waveChanged()); //is ..
+    }
 }
 
 
 /*!
-    \fn SoundManager::lockedBeatLinesChanged(QValueList<long> beatlines)
+    \fn SoundManager::lockedBeatLinesChanged(QValueList<beatline_data> beatlines)
  */
-void SoundManager::lockedBeatLinesChanged(QValueList<long> beatlines)
+void SoundManager::lockedBeatLinesChanged(QValueList<beatline_data> beatlines)
 {
     lockedBeatLines=beatlines;
     applyEnvelope();
@@ -1767,9 +1876,9 @@ void SoundManager::lockedBeatLinesChanged(QValueList<long> beatlines)
 
 
 /*!
-    \fn SoundManager::beatLinesChanged(QValueList<long> beatlines)
+    \fn SoundManager::beatLinesChanged(QValueList<beatline_data> beatlines)
  */
-void SoundManager::beatLinesChanged(QValueList<long> beatlines)
+void SoundManager::beatLinesChanged(QValueList<beatline_data> beatlines)
 {
     beatLines=beatlines;
 }
@@ -1790,7 +1899,7 @@ void SoundManager::envelopeChanged(EnvelopeData* envData)
             envelopeArray[i]=envelopeData->getEnvAt(float(i)/float(applyNbFrames),
                                             100.0/float(applyNbFrames));
         }
-        applyEnvelope();
+        if(!loadingProject)applyEnvelope();
     }
 }
 
@@ -1839,7 +1948,50 @@ bool SoundManager::canOpen(QString file)
     bool openable=FALSE;
     SF_INFO tempsfinfo;
     SNDFILE* tmp_sndfile= sf_open(file, SFM_READ, &tempsfinfo);
-    if(tmp_sndfile==0) openable=FALSE; else openable=TRUE;
+    if(tmp_sndfile) openable=TRUE; else openable=FALSE;
     sf_close(tmp_sndfile);
     return openable;
 }
+
+
+/*!
+    \fn SoundManager::insertFileAt(QString filename,long start_sample)
+ */
+long SoundManager::insertFileAt(QString filename,long start_sample)
+{
+    long amount=SoundHolder::insertFileAt(filename,start_sample);
+    zapInstantEBuffer();
+    if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); 
+    else if(getHasEnvelope()) applyEnvelope();
+    else emit(waveChanged());
+    emit(inserted(start_sample, amount)); //not used
+    return amount;
+}
+
+
+
+/*!
+    \fn SoundManager::replaceFileAt(QString filename, long start, long end)
+ */
+void SoundManager::replaceFileAt(QString filename, long start, long end)
+{
+    SoundHolder::replaceFileAt(filename,start,end);
+    zapInstantEBuffer();
+    if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); 
+    else if(getHasEnvelope()) applyEnvelope();
+    else emit(waveChanged());
+    emit(replaced(start, end)); //not used
+}
+
+
+
+
+
+bool SoundManager::getLoadingProject() const {
+    return loadingProject;
+}
+
+
+void SoundManager::setLoadingProject(bool theValue) {
+    loadingProject = theValue;
+}
