@@ -73,61 +73,69 @@ SoundPlayer::SoundPlayer()
         bufferLR=new short[buffer_len*2];
         bufferPointerL=0;
         bufferPointerR=0;
-        devdsp_audio_device=0;
+        #ifdef HAS_OSS
+            devdsp_audio_device=0;
+        #endif
         leftLoop=0;
         rightLoop=0;
         output=OUTPUT_NONE;
         playFromFileV=FALSE;
-        previewFileJack=0;
+        previewFile=0;
         savedLeftLoop=-1;
         savedRightLoop=-1;
         closeV=FALSE;
+        #ifdef HAS_PORTAUDIO
+            portaudio_stream=0;
+        #endif
+        processedPositions=0;
 }
 
 
 SoundPlayer::~SoundPlayer()
 {
     zaparr(bufferLR);
-    if(output==OUTPUT_DEVDSP) ::close (devdsp_audio_device) ;
-    else if(output==OUTPUT_JACK) closeJack();
+    #ifdef HAS_OSS
+        if(output==OUTPUT_DEVDSP) ::close (devdsp_audio_device) ;
+    #endif
+    #ifdef HAS_JACK
+        if(output==OUTPUT_JACK) closeJack();
+    #endif
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) closePortaudio();
+    #endif
 }
-
+#ifdef HAS_OSS
 void SoundPlayer::playDevDsp(){
-        if(output==OUTPUT_DEVDSP){
-            while(position<rightLoop && playing){
-                    if(position>=0 && position<soundManager->getFrames()){
-                        if(soundManager->isPlayingBandOnly()){
-                                soundManager->exportMultiplexedShortChannelBand(bufferLR,position,position+buffer_len);
-                        	position+=buffer_len;
-			}else{
-				if(position+buffer_len<rightLoop){
-                                	soundManager->exportMultiplexedShortBuffer(bufferLR, position,position+buffer_len);
-                                       if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, buffer_len*2);
-					write (devdsp_audio_device,bufferLR,buffer_len*2* sizeof (short)) ;
-					position+=buffer_len;
-                                }else if (position<rightLoop){
-					soundManager->exportMultiplexedShortBuffer(bufferLR, position,rightLoop);
-                                        if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, (rightLoop-position)*2);
-					write (devdsp_audio_device,bufferLR,(rightLoop-position)*2* sizeof (short));
-					position=rightLoop;
-                                }else position=rightLoop;
-			}
-                        positionChanged(position);
-                    }
-            }
-            if(playingOnce){
-                this->stopPlay();
-                playingOnce=FALSE;
-                resetLoop();
-            }
+        while(position<rightLoop && playing){
+            if(position+buffer_len<rightLoop){
+                soundManager->exportMultiplexedShortBuffer(bufferLR, position,position+buffer_len);
+                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, buffer_len*2);
+                    write (devdsp_audio_device,bufferLR,buffer_len*2* sizeof (short)) ;
+                    position+=buffer_len;
+            }else if (position<rightLoop){
+                soundManager->exportMultiplexedShortBuffer(bufferLR, position,
+                        rightLoop);
+                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, (rightLoop-position)*2);
+                    write (devdsp_audio_device,bufferLR,(rightLoop-position)*2* sizeof (short));
+                    position=rightLoop;
+            }else position=rightLoop;
+            positionChanged(position);
+        }
+        if(playingOnce){
+            this->stopPlay();
+            playingOnce=FALSE;
+            resetLoop();
         }
 }
-
+#endif
 void SoundPlayer::stopPlay(){
 
         if(playing){
             playing=false;
             positionChanged(leftLoop);
+            #ifdef HAS_PORTAUDIO
+                if(output==OUTPUT_PORTAUDIO) stopPAStream();
+            #endif
         }
 }
 
@@ -147,6 +155,13 @@ void SoundPlayer::setSoundmanager(SoundManager *sm)
 
 void SoundPlayer::positionChanged(long _pos)
 {
+    /*
+    if(processedPositions>=1000){
+        processedPositions=0;
+        mainApplication->processEvents(1);
+    }
+    */
+    processedPositions++;
     position=_pos;
     QObject* soundMonitor;
     for ( soundMonitor = soundMonitors.first(); soundMonitor; soundMonitor = soundMonitors.next() ){
@@ -159,6 +174,24 @@ void SoundPlayer::positionChanged(long _pos)
     }
 
 }
+#ifdef HAS_OSS
+/*!
+    \fn SoundPlayer::connect_devdsp()
+ */
+int SoundPlayer::connect_devdsp()
+{
+        fprintf (stderr, "Querying /dev/dsp.\n");
+        //devdsp_audio_device = linux_open_dsp_device (soundManager->getChannels(),
+        //                                                                soundManager->getRate()) ;
+        devdsp_audio_device = linux_open_dsp_device (2,44100) ;
+        if(devdsp_audio_device){
+            output=OUTPUT_DEVDSP;
+            fprintf (stderr, "Connected to /dev/dsp.\n");
+            return 0;
+            bufferLR=new short[buffer_len*2];
+        }else return 1;
+}
+
 
 int SoundPlayer::linux_open_dsp_device (int channels, int srate)
 {	int fd, stereo, temp, error ;
@@ -201,14 +234,14 @@ int SoundPlayer::linux_open_dsp_device (int channels, int srate)
 
         return 	fd ;
 } /* linux_open_dsp_device */
-
+#endif
 
 /*!
     \fn SoundPlayer::run
  */
 void SoundPlayer::run()
 {
-
+    #ifdef HAS_OSS
     if(output==OUTPUT_DEVDSP){
         while(!closeV){
             if(playing){
@@ -222,6 +255,7 @@ void SoundPlayer::run()
             }
         }
     }
+    #endif
 }
 
 
@@ -252,6 +286,10 @@ bool SoundPlayer::isPlaying()
 void SoundPlayer::setPlaying(bool _pl)
 {
     playing=_pl;
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO && playing) startPAStream();
+        else if(output==OUTPUT_PORTAUDIO && !playing) stopPAStream();
+    #endif
 }
 
 
@@ -262,7 +300,8 @@ void SoundPlayer::setLoop(long l,long r)
 {
     leftLoop=l<0?0:l;
     rightLoop=r>=soundManager->getFrames()?soundManager->getFrames()-1:r;
-    if(!playing) position=leftLoop;
+    position=leftLoop;
+    //cout<<"position set in setLoop("<<l<<","<<r<<") at "<<position<<endl;
 }
 
 /*!
@@ -274,7 +313,8 @@ void SoundPlayer::setOneTimeLoop(long l,long r)
     savedRightLoop=rightLoop;
     leftLoop=l;
     rightLoop=r;
-    if(!playing) position=leftLoop;
+    position=leftLoop;
+    //cout<<"position set in setOneTimeLoop("<<l<<","<<r<<") at "<<position<<endl;
 }
 
 
@@ -286,6 +326,11 @@ void SoundPlayer::setPlayingOnce(bool po)
     playingOnce=po;
 }
 
+
+
+
+
+#ifdef HAS_JACK
 int SoundPlayer::closeJack ()
 {
     jack_deactivate (jack_handle);
@@ -303,7 +348,7 @@ int SoundPlayer::connect_jack()
 
 
     std::cout<<"Querying Jack...\n";
-    QString jackName="Freecycle";
+    QString jackName="Freecycle "+QString::number(QDateTime::currentDateTime().toTime_t());
     if ((jack_handle = jack_client_new (jackName.latin1())) == 0){
         fprintf (stderr, "Can't connect to JACK\n");
         return 1;
@@ -315,8 +360,8 @@ int SoundPlayer::connect_jack()
     //periodsize = MAXIMUM_PERIODSIZE;
 
 
-        jack_out [0] = jack_port_register (jack_handle, "out L", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-        jack_out [1] = jack_port_register (jack_handle, "out R", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+    jack_out [0] = jack_port_register (jack_handle, "out L", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+    jack_out [1] = jack_port_register (jack_handle, "out R", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 
 
     if (jack_activate (jack_handle))
@@ -337,22 +382,6 @@ int SoundPlayer::connect_jack()
 }
 
 
-/*!
-    \fn SoundPlayer::connect_devdsp()
- */
-int SoundPlayer::connect_devdsp()
-{
-        fprintf (stderr, "Querying /dev/dsp.\n");
-        //devdsp_audio_device = linux_open_dsp_device (soundManager->getChannels(),
-        //                                                                soundManager->getRate()) ;
-        devdsp_audio_device = linux_open_dsp_device (2,44100) ;
-        if(devdsp_audio_device){
-            output=OUTPUT_DEVDSP;
-            fprintf (stderr, "Connected to /dev/dsp.\n");
-            return 0;
-            bufferLR=new short[buffer_len*2];
-        }else return 1;
-}
 
 int SoundPlayer::jack_static_callback (jack_nframes_t nframes, void *arg)
 {
@@ -365,17 +394,22 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
                  (jack_default_audio_sample_t *)(jack_port_get_buffer (jack_out [0], nframes));
     jack_default_audio_sample_t * p2 =
                  (jack_default_audio_sample_t *)(jack_port_get_buffer (jack_out [1], nframes));
-    if(!soundManager) return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
+    if(!(p1 && p2)){
+        cout<<"Jack buffers unavailable!!\n";
+        resetLoop();
+        playing=FALSE;
+        playingOnce=FALSE;
+        return 0;
+    }
     if(nframes>buffer_len) setJackBuffersLen(nframes);
-
     if(playFromFileV){
-        if (!previewFileJack) return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
+        if (!previewFile) return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
         float* buffL=new float[nframes];
         float* buffR=new float[nframes];
         float* buff=new float[nframes*2];
-        long length=sfinfoForJackPreview.frames;
-        if(position>=0 && position+buffer_len<=length){
-            sf_readf_float(previewFileJack, buff,(sf_count_t)(nframes));
+        long length=sfinfoForPreview.frames;
+        if(position>=0 && position+buffer_len<length){
+            sf_readf_float(previewFile, buff,(sf_count_t)(nframes));
             int count=0;
             for (int i=0;i<nframes;i++){
                 buffL[i]=buff[count++];
@@ -394,56 +428,42 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
         zaparr(buffR);
         zaparr(buff);
     }else{
+        if(!soundManager || !soundManager->hasSound()) 
+            return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
         if(isPlaying()){
-                if(position>=rightLoop || position>=soundManager->getFrames()){
-                    position=leftLoop;
+            Q_ASSERT(position<soundManager->getFrames());
+            Q_ASSERT(position>=0);
+            bufferPointerL=&soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[position];
+            bufferPointerR=&soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[position];
+            if(position+nframes<getRightLoop()){
+                memcpy (p1, bufferPointerL, sizeof(jack_default_audio_sample_t) * nframes);
+                memcpy (p2, bufferPointerR, sizeof(jack_default_audio_sample_t) * nframes);
+                position+=nframes;
+            }else{
+                    memcpy (p1, bufferPointerL,
+                        sizeof(jack_default_audio_sample_t) * (getRightLoop()-position));
+                    memcpy (p2, bufferPointerR,
+                        sizeof(jack_default_audio_sample_t) * (getRightLoop()-position));
                     if(playingOnce){
-                        this->stopPlay();
+                        setJackNotPlaying(&(p1[getRightLoop()-position]),nframes-(getRightLoop()-position));
+                        setJackNotPlaying(&(p2[getRightLoop()-position]),nframes-(getRightLoop()-position));
+                        playing=FALSE;//this->stopPlay();
                         playingOnce=FALSE;
-                        return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
-                    }
-                }
-                if(soundManager->isPlayingBandOnly()){
-                    float* outputLeft=new float[nframes];
-                    float* outputRight=new float[nframes];
-                    soundManager->getLeftChannelBand(outputLeft,position,position+nframes);
-                    soundManager->getRightChannelBand(outputRight,position,position+nframes);
-
-                    if (outputLeft) memcpy (p1, outputLeft, sizeof(jack_default_audio_sample_t) * nframes);
-                    else   setJackNotPlaying(p1,nframes);
-
-                    if (outputRight) memcpy (p2, outputRight, sizeof(jack_default_audio_sample_t) * nframes);
-                    else   setJackNotPlaying(p2,nframes);
-
-                    zaparr(outputLeft);
-                    zaparr(outputRight);
-                }else{
-                    bufferPointerL=&soundManager->getLeftChannel()[position];
-                    bufferPointerR=&soundManager->getRightChannel()[position];
-
-                    if(position+nframes<rightLoop){
-                        if (bufferPointerL) memcpy (p1, bufferPointerL, sizeof(jack_default_audio_sample_t) * nframes);
-                        else   setJackNotPlaying(p1,nframes);
-                        if (bufferPointerR) memcpy (p2, bufferPointerR, sizeof(jack_default_audio_sample_t) * nframes);
-                        else   setJackNotPlaying(p2,nframes);
-                        position+=nframes;
+                        position=leftLoop;
+                        positionChanged(position);
+                        resetLoop();
+                        return 0;
                     }else{
-                         if (bufferPointerL){
-                             memcpy (p1, bufferPointerL, sizeof(jack_default_audio_sample_t) * (rightLoop-position));
-                             memcpy (&p1[rightLoop-position],
-                                 &soundManager->getLeftChannel()[leftLoop],
-                                 sizeof(jack_default_audio_sample_t) * (nframes-(rightLoop-position)));
-                         }else   setJackNotPlaying(p1,nframes);
-                        if (bufferPointerR){
-                            memcpy (p2, bufferPointerL, sizeof(jack_default_audio_sample_t) * (rightLoop-position));
-                             memcpy (&p2[rightLoop-position],
-                                 &soundManager->getRightChannel()[leftLoop],
-                                 sizeof(jack_default_audio_sample_t) * (nframes-(rightLoop-position)));
-                        }else   setJackNotPlaying(p2,nframes);
-                        position=leftLoop+(nframes-(rightLoop-position));
+                        memcpy (&p1[getRightLoop()-position],
+                            &soundManager->getLeftChannel()[getLeftLoop()],
+                            sizeof(jack_default_audio_sample_t) * (nframes-(getRightLoop()-position)));
+                        memcpy (&p2[getRightLoop()-position],
+                            &soundManager->getRightChannel()[getLeftLoop()],
+                            sizeof(jack_default_audio_sample_t) * (nframes-(getRightLoop()-position)));
+                        position=getLeftLoop()+(nframes-(getRightLoop()-position));
                     }
-                }
-                positionChanged(position);
+            }
+            positionChanged(position);
         }else{
             setJackNotPlaying(p1,nframes);
             setJackNotPlaying(p2,nframes);
@@ -453,13 +473,34 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
 }
 
 
+/*!
+    \fn SoundPlayer::setJackNotPlaying()
+ */
+int SoundPlayer::setJackNotPlaying(jack_default_audio_sample_t * p, jack_nframes_t nframes)
+{
+    memset (p, 0, sizeof(jack_default_audio_sample_t) * nframes);
+    return 0;
+}
+
+#endif
+
 /*!
     \fn SoundPlayer::init()
  */
 void SoundPlayer::init()
 {
 
-    if(connect_jack() && connect_devdsp()){
+    if(
+       #ifdef HAS_JACK
+        connect_jack() &&
+       #endif
+       #ifdef HAS_PORTAUDIO 
+        connect_portaudio() &&
+       #endif
+       #ifdef HAS_OSS
+        connect_devdsp() &&
+       #endif
+        true){
         output=OUTPUT_NONE;
         std::cout<<"Oh, anyways we need no sound..Aren't we?\n";
     }
@@ -498,14 +539,6 @@ void SoundPlayer::cropped(long start,long end){
 }
 
 
-/*!
-    \fn SoundPlayer::setJackNotPlaying()
- */
-int SoundPlayer::setJackNotPlaying(jack_default_audio_sample_t * p, jack_nframes_t nframes)
-{
-    memset (p, 0, sizeof(jack_default_audio_sample_t) * nframes);
-    return 0;
-}
 
 
 /*!
@@ -534,7 +567,11 @@ void SoundPlayer::playFromFile(QString path)
     playFromFileV=TRUE;
     playFromFilePath=path;
     //position=0;
-    if(output==OUTPUT_JACK)  previewFileJack= sf_open(path, SFM_READ, &sfinfoForJackPreview);
+    if(output==OUTPUT_JACK || output==OUTPUT_PORTAUDIO) 
+        previewFile= sf_open(path, SFM_READ, &sfinfoForPreview);
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) startPAStream();
+    #endif
 }
 
 /*!
@@ -549,6 +586,7 @@ void SoundPlayer::playFromFileThreaded()
     if (!tmp_sndfile) return;
     sf_seek (tmp_sndfile, (sf_count_t)position, SEEK_SET);
     length=sfinfo.frames;
+    #ifdef HAS_OSS
     if(output==OUTPUT_DEVDSP){
             while(position<length && playFromFileV){
                 if(position>0 && position+buffer_len<=length){
@@ -563,6 +601,7 @@ void SoundPlayer::playFromFileThreaded()
             sf_close(tmp_sndfile);
             if(playFromFileV) stopPlayFromFile();
         }
+    #endif
 }
 
 
@@ -575,7 +614,10 @@ void SoundPlayer::stopPlayFromFile()
     playing=FALSE;
     position=0;
     positionChanged(position);
-    if(output==OUTPUT_JACK) sf_close(previewFileJack);
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) stopPAStream();
+    #endif
+    if(output==OUTPUT_JACK || output==OUTPUT_PORTAUDIO) sf_close(previewFile);
 }
 
 /*!
@@ -584,7 +626,10 @@ void SoundPlayer::stopPlayFromFile()
 void SoundPlayer::pausePlayFromFile()
 {
     playFromFileV=FALSE;
-    //if(output==OUTPUT_JACK) sf_close(previewFileJack);
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) stopPAStream(); 
+    #endif
+    //if(output==OUTPUT_JACK) sf_close(previewFile);
 }
 
 
@@ -608,6 +653,9 @@ void SoundPlayer::resetLoop()
  */
 void SoundPlayer::setPosition(long pos)
 {
+    if (playFromFileV){
+        sf_seek (previewFile, (sf_count_t)pos, SEEK_SET);
+    }
     positionChanged(pos);
 }
 
@@ -633,4 +681,198 @@ void SoundPlayer::removeSoundMonitor(QObject* sm)
     }
 }
 
+#ifdef HAS_PORTAUDIO
+/*!
+    \fn SoundPlayer::portaudio_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long framesPerBuffer,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags)
+ */
+int SoundPlayer::portaudio_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long nframes,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags)
+{
+    if(!soundManager) return paComplete;
+    if(playFromFileV){
+        if (!previewFile) return paComplete;
+        long length=sfinfoForPreview.frames;
+        //cout<<"position: "<<position<<endl;
+        if(position>=0 && position+nframes<length){
+            //cout<<"position: "<<position<<endl;
+            sf_readf_float(previewFile, (float*)outputBuffer,(sf_count_t)(nframes));
+            position+=nframes;
+            positionChanged(position);
+            return paContinue;
+        }else{
+            sf_readf_float(previewFile, (float*)outputBuffer,(sf_count_t)(length-position));
+            memset ((&((float*)outputBuffer)[length-position]), 0,
+                                 sizeof(float) * (nframes-(length-position))*2);
+            playFromFileV=FALSE;//stopPlayFromFile();
+            return paAbort;
+        }
+    }else{
+        if(isPlaying()){
+                if(position+nframes<getRightLoop()){
+                
+                    soundManager->exportMultiplexedBuffer((float*)outputBuffer,
+                                            position,
+                                            position+long(nframes));
+                    position+=nframes;
+                }else{
+                    soundManager->exportMultiplexedBuffer((float*)outputBuffer,
+                                            position,
+                                            getRightLoop());
+                    if(playingOnce){
+                        memset ((&((float*)outputBuffer)[getRightLoop()-position]), 0,
+                                 sizeof(float) * (nframes-(getRightLoop()-position))*2);
+                        playing=FALSE; //stopPlay(); 
+                        playingOnce=FALSE;
+                        position=leftLoop;
+                        positionChanged(position);
+                        resetLoop(); 
+                        return paAbort;
+                    }else{
+                        soundManager->exportMultiplexedBuffer(
+                                                (&((float*)outputBuffer)[getRightLoop()-position]),
+                                                getLeftLoop(),
+                                                getLeftLoop()+(nframes-(getRightLoop()-position)));
+                        position=getLeftLoop()+(nframes-(getRightLoop()-position));
+                    }
+                }
+                positionChanged(position);
+                return paContinue;
+        }else{
+            return paComplete;
+        }
+    }
+    return paComplete;
+    
+}
 
+/*!
+    \fn SoundPlayer::portaudio_static_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long framesPerBuffer,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags, void *userData )
+ */
+int SoundPlayer::portaudio_static_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long framesPerBuffer,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags, void *userData )
+{
+    return ((SoundPlayer *) userData)->portaudio_callback (inputBuffer,
+                                                           outputBuffer,
+                                                           framesPerBuffer,
+                                                           timeInfo,
+                                                           statusFlags);
+}
+
+
+/*!
+    \fn SoundPlayer::connect_portaudio()
+ */
+int SoundPlayer::connect_portaudio()
+{
+    std::cout<<"Querying Portaudio...\n";
+    PaError err=Pa_Initialize();
+    if( err != paNoError ){
+        printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        return 1;
+    }
+    
+    
+    
+    err = Pa_OpenDefaultStream(
+                    &portaudio_stream,        /* passes back stream pointer */
+                    0,              /* no input channels */
+                    2,              /* stereo output */
+                    paFloat32,      /* 32 bit floating point output */
+                    44100,          /* sample rate */
+                    256,            /* frames per buffer */
+                    portaudio_static_callback, /* specify our custom callback */
+                    (void *)this );        /* pass our data through to callback */
+    
+    if( err != paNoError ){
+        printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        return 1;
+    }
+    output=OUTPUT_PORTAUDIO;
+    fprintf (stderr, "Connected to Portaudio.\n");
+    return 0;
+}
+
+
+/*!
+    \fn SoundPlayer::closePortaudio()
+ */
+void SoundPlayer::closePortaudio()
+{
+    Pa_CloseStream(portaudio_stream);
+    Pa_Terminate();
+}
+
+
+/*!
+    \fn SoundPlayer::startPAStream()
+ */
+int SoundPlayer::startPAStream()
+{
+    if(Pa_IsStreamActive(portaudio_stream)){
+        Pa_StopStream(portaudio_stream);
+    }
+    if(Pa_IsStreamStopped( portaudio_stream )==1){
+        PaError err=Pa_StartStream( portaudio_stream );
+        if( err != paNoError ){
+            printf(  "(in start) PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        }else{
+            //cout<<"Latency: "<<Pa_GetStreamInfo( portaudio_stream )->outputLatency<<"sec"<<endl;
+        }
+        return err;
+    } return 0;
+}
+
+
+/*!
+    \fn SoundPlayer::stopPAStream()
+ */
+int SoundPlayer::stopPAStream()
+{
+    if(Pa_IsStreamStopped( portaudio_stream )!=1){
+        PaError err=Pa_AbortStream( portaudio_stream );
+        if( err != paNoError ){
+            printf(  "(in stop) PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        }else{
+            //while(portaudio_stream){}
+        }
+        return err;
+    }
+    return 0;
+}
+#endif
+
+long SoundPlayer::getRightLoop() const {
+    if (position<rightLoop) return rightLoop;
+    else return soundManager->getFrames();
+}
+
+
+void SoundPlayer::setRightLoop(const long& theValue) {
+    rightLoop = theValue;
+}
+
+
+long SoundPlayer::getLeftLoop() const {
+    if (position<rightLoop) return leftLoop;
+    else return 0;
+}
+
+
+void SoundPlayer::setLeftLoop(const long& theValue) {
+    leftLoop = theValue;
+}
+
+
+void SoundPlayer::setMainApplication(QApplication* theValue) {
+    mainApplication = theValue;
+}
