@@ -167,7 +167,7 @@ void SplineLine::moveSelectedPoint(QPoint to,long sample_pos, float value)
     for (int i=0;i<splinePoints.count();i++){
 
         if(splinePoints[i].selected()){
-            if((splinePoints.contains(SplinePoint(to))<=1) && !containsSameX(to)){
+            if((splinePoints.contains(SplinePoint(to))<=1) && containsSameX(to)<=1){
                 int diffx=splinePoints[i].x()-to.x();
                 int diffy=splinePoints[i].y()-to.y();
                 long diff_sample=splinePoints[i].getSamplePos()-sample_pos;
@@ -208,7 +208,7 @@ void SplineLine::moveSelectedPoint(QPoint to,long sample_pos, float value)
     }
     qHeapSort(splinePoints);
     toSortablePoints();
-    if(points.first()==to || points.last()==to){
+    if(sortablePoints.first()==to || sortablePoints.last()==to){
             canvas()->setAllChanged();
         }
     setPoints(toPointArray());
@@ -271,7 +271,7 @@ void SplineLine::updateSize(QSize size)
     qHeapSort(splinePoints);
     setPoints(toPointArray());
     toSortablePoints();
-    setX(points[0].x());
+    setX(sortablePoints[0].x());
     canvas()->setAllChanged();
     update();
 }
@@ -331,7 +331,7 @@ int SplineLine::count()
 void SplineLine::clear()
 {
     splinePoints.clear();
-    points.clear();
+    sortablePoints.clear();
     canvas()->setAllChanged();
     update();
 }
@@ -342,10 +342,10 @@ void SplineLine::clear()
 QPointArray SplineLine::toPointArray()
 {
     toSortablePoints();
-    QPointArray array(points.count());
-    qHeapSort(points);
-    for (int i=0;i<points.count();i++){
-        array.setPoint(i,QPoint(points[i].x(),points[i].y()));
+    QPointArray array(sortablePoints.count());
+    qHeapSort(sortablePoints);
+    for (int i=0;i<sortablePoints.count();i++){
+        array.setPoint(i,QPoint(sortablePoints[i].x(),sortablePoints[i].y()));
     }
     return array;
 }
@@ -355,14 +355,14 @@ QPointArray SplineLine::toPointArray()
  */
 void SplineLine::toSortablePoints()
 {
-    points.clear();
+    sortablePoints.clear();
     int count=0;
     for (int i=0;i<splinePoints.count();i++){
-        points.append(splinePoints[i]);
-        if(i!=0) points.append(splinePoints[i].cp1);
-        if(i!=splinePoints.count()-1) points.append(splinePoints[i].cp2);
+        sortablePoints.append(splinePoints[i]);
+        if(i!=0) sortablePoints.append(splinePoints[i].cp1);
+        if(i!=splinePoints.count()-1) sortablePoints.append(splinePoints[i].cp2);
     }
-    qHeapSort(points);
+    qHeapSort(sortablePoints);
 }
 
 
@@ -419,3 +419,100 @@ int SplineLine::valToYPos(float val)
 {
     return int(MyMath::linear_regression(minValue,maxValue,canvas()->height(),0,val));
 }
+
+
+/*!
+    \fn SplineLine::crop(long start, long end)
+ */
+void SplineLine::crop(long start, long end)
+{
+    cout<<"cropping spline\n";
+     //first move boundaries
+    for (int i=0;i<splinePoints.count();i++){
+        if(splinePoints[i].getSamplePos()<start && 
+            i<splinePoints.count()-1 && splinePoints[i+1].getSamplePos()>start){
+                        long next_sample_pos=splinePoints[i+1].getSamplePos();
+                        float next_value=splinePoints[i+1].getValue();
+                        long old_sample_pos=splinePoints[i].getSamplePos();
+                        float old_value=splinePoints[i].getValue();
+                        long new_sample_pos=start;
+                        float new_value=MyMath::linear_regression(old_sample_pos,next_sample_pos,
+                                                              old_value,next_value,new_sample_pos);
+                        splinePoints[i].setSamplePos(new_sample_pos);
+                        splinePoints[i].setValue(new_value);
+                        
+                        next_sample_pos=splinePoints[i+1].cp1.getSamplePos();
+                        next_value=splinePoints[i+1].cp1.getValue();
+                        old_sample_pos=splinePoints[i].cp1.getSamplePos();
+                        old_value=splinePoints[i].cp1.getValue();
+                        new_sample_pos=
+                            start+(splinePoints[i].getSamplePos()-splinePoints[i].cp1.getSamplePos());
+                        new_value=MyMath::linear_regression(old_sample_pos,next_sample_pos,
+                                                              old_value,next_value,new_sample_pos);
+                        splinePoints[i].cp1.setSamplePos(new_sample_pos);
+                        splinePoints[i].cp1.setValue(new_value);
+                        
+                        next_sample_pos=splinePoints[i+1].cp2.getSamplePos();
+                        next_value=splinePoints[i+1].cp2.getValue();
+                        old_sample_pos=splinePoints[i].cp2.getSamplePos();
+                        old_value=splinePoints[i].cp2.getValue();
+                        new_sample_pos=
+                            start+(splinePoints[i].getSamplePos()-splinePoints[i].cp2.getSamplePos());
+                        new_value=MyMath::linear_regression(old_sample_pos,next_sample_pos,
+                                                              old_value,next_value,new_sample_pos);
+                        splinePoints[i].cp2.setSamplePos(new_sample_pos);
+                        splinePoints[i].cp2.setValue(new_value);
+        }
+        if(splinePoints[i].getSamplePos()>end && i>1 && splinePoints[i-1].getSamplePos()<end){
+                        long previous_sample_pos=splinePoints[i-1].getSamplePos();
+                        float previous_value=splinePoints[i-1].getValue();
+                        long old_sample_pos=splinePoints[i].getSamplePos();
+                        float old_value=splinePoints[i].getValue();
+                        long new_sample_pos=end;
+                        float new_value=MyMath::linear_regression(previous_sample_pos,old_sample_pos,
+                                                              previous_value,old_value,new_sample_pos);
+                        splinePoints[i].setSamplePos(new_sample_pos);
+                        splinePoints[i].setValue(new_value);
+                        
+                        previous_sample_pos=splinePoints[i-1].cp1.getSamplePos();
+                        previous_value=splinePoints[i-1].cp1.getValue();
+                        old_sample_pos=splinePoints[i].cp1.getSamplePos();
+                        old_value=splinePoints[i].cp1.getValue();
+                        new_sample_pos=
+                            end-(splinePoints[i].cp1.getSamplePos()-splinePoints[i].getSamplePos());
+                        new_value=MyMath::linear_regression(previous_sample_pos,old_sample_pos,
+                                                              previous_value,old_value,new_sample_pos);
+                        splinePoints[i].cp1.setSamplePos(new_sample_pos);
+                        splinePoints[i].cp1.setValue(new_value);
+                        
+                        previous_sample_pos=splinePoints[i-1].cp2.getSamplePos();
+                        previous_value=splinePoints[i-1].cp2.getValue();
+                        old_sample_pos=splinePoints[i].cp2.getSamplePos();
+                        old_value=splinePoints[i].cp2.getValue();
+                        new_sample_pos=
+                            end-(splinePoints[i].cp2.getSamplePos()-splinePoints[i].getSamplePos());
+                        new_value=MyMath::linear_regression(previous_sample_pos,old_sample_pos,
+                                                              previous_value,old_value,new_sample_pos);
+                        splinePoints[i].cp2.setSamplePos(new_sample_pos);
+                        splinePoints[i].cp2.setValue(new_value);
+        }
+    }
+    qHeapSort(splinePoints);
+    //then remove before and after and change space
+    typedef QValueList <SplinePoint> SplinePointList;
+    SplinePointList::iterator it=splinePoints.begin();
+    while( it != splinePoints.end()){
+        if((*it).getSamplePos()<start){
+                    it=splinePoints.remove(it);
+        }
+        else if((*it).getSamplePos()>end){
+                    it=splinePoints.remove(it);
+        }else{
+            (*it).setSamplePos((*it).getSamplePos()-start);
+            (*it).cp1.setSamplePos((*it).cp1.getSamplePos()-start);
+            (*it).cp2.setSamplePos((*it).cp2.getSamplePos()-start);
+            it++;
+        }
+    }
+    updateSize(canvas()->size());
+}
