@@ -59,6 +59,8 @@ protected:
     float maxValue;
     float minValue;
     void toSortablePoints();
+public slots:
+    void crop(long start, long end);
 };
 
 #endif
