@@ -18,6 +18,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include "wavecanvas.h"
+#include "rms.h"
 /**
         \fn WaveCanvas::WaveCanvas(QWidget* parent)
         Initialises member variables
@@ -39,16 +40,27 @@ WaveCanvas::WaveCanvas(QWidget* parent)
     hasPixmapV=FALSE;
     leftMaximaBuffer=0;
     rightMaximaBuffer=0;
+    leftRMSBuffer=0;
+    rightRMSBuffer=0;
     spectrogramMaxBuffer=0;
     spectrogramMaxBufferNbValues=0;
     maxBufferNbValues=0;
     visibleWidth=0;
+    rmsL=rms_env_new();
+    rmsR=rms_env_new();
 }
 
 
 WaveCanvas::~WaveCanvas()
 {
     //zapBuffers();
+    QCanvasItemList all=allItems();
+    QCanvasItemList::iterator it;
+    for ( it = all.begin(); it != all.end(); ++it ){
+        (*it)->hide();
+    }
+    rms_env_free(rmsL);
+    rms_env_free(rmsR);
 }
 
 /**
@@ -61,12 +73,9 @@ WaveCanvas::~WaveCanvas()
         \sa drawBackground()
 */
 void WaveCanvas::drawBackgroundImage(long start_sample, long end_sample){
-    QMutex mutex;
-    mutex.lock();
     //cout<<"in wavecanvas (parent ["<<parent()->parent()->name()<<"]) drawBackgroundImage()\n";
     if(viewMode==SPECTRUM_VIEWMODE) drawSpectrum(start_sample,end_sample);
     else drawWave(start_sample,end_sample);
-    mutex.unlock();
 }
 
 /**
@@ -79,8 +88,6 @@ void WaveCanvas::drawBackgroundImage(long start_sample, long end_sample){
         \sa drawBackgroundImage(long,long)
 */
 void WaveCanvas::drawBackground( QPainter & localpainter, const QRect & clip ){
-        QMutex mutex;
-        mutex.lock();
         if(soundManager!=0 && soundManager->hasSound()){
             if(backgroundPix){
                 localpainter.drawPixmap(clip.x(),0,
@@ -89,7 +96,6 @@ void WaveCanvas::drawBackground( QPainter & localpainter, const QRect & clip ){
                                                     clip.width(),backgroundPix->height());
             }
         }
-        mutex.unlock();
 }
 
 
@@ -164,11 +170,13 @@ void WaveCanvas::forceRepaint()
     if(backgroundPix && backgroundPix->width()>0){
         long start_sample=pixmapXOrigin*zoomRatio;
         long end_sample= start_sample+visibleWidth*zoomRatio;
-        //cout<<"S: "<<start_sample<<" E: "<<end_sample<<" PX: "<<pixmapXOrigin<<" Z: "<<zoomRatio<<endl;
+        
+        cout<<"S: "<<start_sample<<" E: "<<end_sample<<" PX: "<<pixmapXOrigin<<" Z: "<<zoomRatio<<endl;
         /*
         assert(start_sample>=0 &&
                 end_sample<=soundManager->getFrames() && end_sample>start_sample);
         */
+        end_sample=end_sample>=soundManager->getFrames()?soundManager->getFrames():end_sample;
         forceRepaint(start_sample,end_sample);
     }else{
         forceRepaint(0,soundManager->getFrames());
@@ -225,13 +233,17 @@ void WaveCanvas::changeViewMode(int vm)
  */
 void WaveCanvas::drawWave(long start_sample,long end_sample)
 {
-    /*
-    QProgressDialog progress("Drawing wave...",NULL,end_sample-start_sample,NULL,"progress",TRUE);
-     progress.setMinimumDuration(3000);
-    */
-    //cout<<"Drawing from "<<start_sample<<" to "<<end_sample<<"\n";
-    if(soundManager!=0 && soundManager->hasSound()&&
+    //QProgressDialog progress("Drawing wave...",NULL,end_sample-start_sample,NULL,"progress",TRUE);
+    //progress.setMinimumDuration(1500);
+    cout<<"Drawing from "<<start_sample<<" to "<<end_sample<<"\n";
+    if(soundManager && soundManager->hasSound()&&
        zoomRatio>=1 && width()>0 && height()>0){
+            
+            float* leftAudio=soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT);
+            float* rightAudio=soundManager->getRightChannel(SoundHolder::FOR_OUTPUT);
+            
+            QPointArray rmsPolygonL;
+            QPointArray rmsPolygonR;
             long jump=zoomRatio;
             if(jump<1) jump=1;
             if(start_sample<0)start_sample=0;
@@ -254,8 +266,12 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
 
 
             QColor wavecolor(50,50,255);
+            QColor rmscolor(255,0,0);
+            QPen wavepen(wavecolor,1);
+            QPen rmspen(rmscolor,1);
+            RasterOp waveRop=localpainter.rasterOp();
+            RasterOp rmsRop=Qt::AndROP;
             int translate=0;
-
             if(_hasRuler /*&& wasEmpty*/){
 
                 QFont font;
@@ -298,7 +314,7 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
             int x=c_x;
             //float sampleHeightFactor=(0.8/SHRT_MAX)*(v_height/4);
             float sampleHeightFactor=0.8*(v_height/4);
-            localpainter.setPen(wavecolor);
+            
             if(!leftMaximaBuffer || !rightMaximaBuffer){
                  //oh some caches..ya know..
                 leftMaximaBuffer=new maxAndMin[(int)ceil(soundManager->getFrames()/jump)];
@@ -309,15 +325,42 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
                 }
                 maxBufferNbValues=(int)ceil(soundManager->getFrames()/jump);
             }
-            for (int i=start_sample;i<=end_sample-jump;i=i+jump){
+            rms_env_reset(rmsL);
+            rms_env_reset(rmsR);
+            bool rmsprefilled=FALSE;
+            int reminder=(end_sample-start_sample) % jump;
+            bool drawRMS=jump<4096;
+            for (int i=start_sample;i<end_sample-reminder;i=i+jump){
                     float maxL=0;
                     float minL=0;
                     float maxR=0;
                     float minR=0;
+                    float maxRMSL=0;
+                    float maxRMSR=0;
+                    float maxRMSLbefore=0;
+                    float maxRMSRbefore=0;
                     if(!leftMaximaBuffer[i/zoomRatio].hasValue ||!rightMaximaBuffer[i/zoomRatio].hasValue){
-                        for (int j=i;j<=i+jump;j++){
-                            float valL=soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[j];
-                            float valR=soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[j];
+                        if(!rmsprefilled && start_sample>RMS_BUF_SIZE){
+                            //prefilling the rms buffer
+                            for (int i2=i-RMS_BUF_SIZE;i2<i;i2++){
+                                float valL=leftAudio[i2];
+                                float valR=rightAudio[i2];
+                                rms_env_process(rmsL,valL);
+                                rms_env_process(rmsR,valR);
+                            }
+                            rmsprefilled=TRUE;
+                        }
+                        float valL=0;
+                        float valR=0;
+                        float rmsvalL=0;
+                        float rmsvalR=0;
+                        for (int j=i;j<i+jump;j++){
+                            valL=leftAudio[j];
+                            valR=rightAudio[j];
+                            if(drawRMS){
+                                rmsvalL=rms_env_process(rmsL,valL);
+                                rmsvalR=rms_env_process(rmsR,valR);
+                            }
                             if(valL>=0){
                                 if(valL>maxL) maxL=valL;
                             }else if(valL<=0){
@@ -328,38 +371,96 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
                             }else if(valR<=0){
                                 if(valR<minR) minR=valR;
                             }
+                            
+                            if(drawRMS){
+                                if(rmsvalL>maxRMSL) maxRMSL=rmsvalL;
+                                if(rmsvalR>maxRMSR) maxRMSR=rmsvalR;
+                            }
                         }
+
                         leftMaximaBuffer[i/jump].min=minL;
                         leftMaximaBuffer[i/jump].max=maxL;
+                        leftMaximaBuffer[i/jump].rms=maxRMSL;
                         leftMaximaBuffer[i/jump].hasValue=1;
+                        
                         rightMaximaBuffer[i/jump].min=minR;
                         rightMaximaBuffer[i/jump].max=maxR;
+                        rightMaximaBuffer[i/jump].rms=maxRMSR;
                         rightMaximaBuffer[i/jump].hasValue=1;
                     }else{
                         minL=leftMaximaBuffer[i/jump].min;
                         maxL=leftMaximaBuffer[i/jump].max;
+                        maxRMSL=leftMaximaBuffer[i/jump].rms;
+                        
                         minR=rightMaximaBuffer[i/jump].min;
                         maxR=rightMaximaBuffer[i/jump].max;
+                        maxRMSR=rightMaximaBuffer[i/jump].rms;
+                        
                     }
+                    maxRMSLbefore=i>0?leftMaximaBuffer[(i-1)/jump].rms:0;
+                    maxRMSRbefore=i>0?rightMaximaBuffer[(i-1)/jump].rms:0;
 
                     //if(hasPixmapV) return; //avoid cascading redraws
-                    localpainter.drawLine(x,translate,x,(int)((float)-maxL*sampleHeightFactor)+translate);
-                    localpainter.drawLine(x,translate,x,(int)((float)-minL*sampleHeightFactor)+translate);
+                    localpainter.setPen(wavepen);
+                    //localpainter.setRasterOp(waveRop);
 
-                    localpainter.drawLine(x,translate+(double)v_height/2.0,
-                                                            x,(int)((float)-maxR*sampleHeightFactor)+translate+(double)v_height/2.0);
-                    localpainter.drawLine(x,translate+(double)v_height/2.0,
-                                                            x,(int)((float)-minR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                    localpainter.drawLine(x,(int)((float)-minL*sampleHeightFactor)+translate,x,(int)((float)-maxL*sampleHeightFactor)+translate);
+                    localpainter.drawLine(x,(int)((float)-minR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                          x,(int)((float)-maxR*sampleHeightFactor)+translate+(double)v_height/2.0);
+
+                    localpainter.setPen(rmspen);
+                    //localpainter.setRasterOp(rmsRop);
+                    if(x==0){
+                        localpainter.drawPoint(x,(int)((float)-maxRMSL*sampleHeightFactor)+translate);
+                        localpainter.drawPoint(x,(int)((float)-maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                        
+                        localpainter.drawPoint(x,(int)((float)+maxRMSL*sampleHeightFactor)+translate);
+                        localpainter.drawPoint(x,(int)((float)+maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                    }else{
+                         /*
+                        rmsPolygonL.putPoints(0,4,
+                                    x,(int)((float)-maxRMSL*sampleHeightFactor)+translate,
+                                    x-1,(int)((float)-maxRMSLbefore*sampleHeightFactor)+translate,
+                                    x,(int)((float)+maxRMSL*sampleHeightFactor)+translate,
+                                    x-1,(int)((float)+maxRMSLbefore*sampleHeightFactor)+translate);
+                                                    
+                        rmsPolygonR.putPoints(0,4,
+                                    x,(int)((float)-maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                    x-1,(int)((float)-maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                    x,(int)((float)+maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                    x-1,(int)((float)+maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0);
+                        
+                        
+                           localpainter.drawPolygon(rmsPolygonL,TRUE);
+                           localpainter.drawPolygon(rmsPolygonR,TRUE);
+                           */
+                        if(drawRMS){
+                            localpainter.drawLine(x,(int)((float)-maxRMSL*sampleHeightFactor)+translate,
+                                            x-1,(int)((float)-maxRMSLbefore*sampleHeightFactor)+translate);
+                            localpainter.drawLine(x,(int)((float)-maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                            x-1,(int)((float)-maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0);
+                        }
+                        
+                        /*
+                        localpainter.drawLine(x,(int)((float)+maxRMSL*sampleHeightFactor)+translate,
+                                        x-1,(int)((float)+maxRMSLbefore*sampleHeightFactor)+translate);
+                        localpainter.drawLine(x,(int)((float)+maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                        x-1,(int)((float)+maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0);
+                          */
+                    }
 
                     x++;
                     /*
-                    if((i-start_sample)%(jump*1000)==0){
+                    if((i-start_sample)%jump==0){
                         progress.setProgress(i);
                         progress.setLabelText("Drawing wave...(frame: "+QString::number(i)+")");
+                        //mainApplication->wakeUpGuiThread();
+                        cout<<"progress\n";
                     }
                     */
             }
             hasPixmapV=TRUE;
+            cout<<"end drawing\n";
     }
 }
 
@@ -383,11 +484,11 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
     float spectralMeanOverFrames=(maximaLog-minimaLog)/2;
     int fftAdvance=soundManager->getHiresFreqFFTAdvance();
 
-
-    QProgressDialog progress("Drawing spectrum...",NULL,end_sample-start_sample,NULL,"progress",TRUE);
+/*
+     QProgressDialog progress("Drawing spectrum...",NULL,end_sample-start_sample,NULL,"progress",TRUE);
      progress.setMinimumDuration(2000);
      progress.setProgress(0);
-
+*/
      if(soundManager!=0 && soundManager->hasSound()&& zoomRatio>=1){
             long jump=zoomRatio;
             if(jump<1) jump=1;
@@ -476,7 +577,8 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
             int lastchunk=end_sample+jump>soundManager->getFrames()?
                                         soundManager->getFrames()%jump:0;
             cout<<"Last chunk: "<<lastchunk<<endl;
-            for (int i=start_sample;i<end_sample-lastchunk;i=i+jump){
+            int remider=(end_sample-start_sample) % jump;
+            for (int i=start_sample;i<end_sample-remider;i=i+jump){
                     bwFactorRound.myround();
                     int y=0;
                     bhFactorRound.init(bandHeightFactor);
@@ -485,6 +587,7 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
                         max=FLT_MIN;
                         if(!spectrogramMaxBuffer[i/jump].hasValue){
                             for (int j=i;j<=i+jump;j=j+fftAdvance){
+                                        //cout<<"nbbands: "<<nbbands<<" j:"<<j<<" fa: "<<fftAdvance<<" band: "<<band<<endl;
                                         val=subband_energies[nbbands-1-band][j/fftAdvance];
                                         if(val>max) max=val;
                             }
@@ -498,8 +601,7 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
                             max=spectrogramMaxBuffer[i/jump].max[band];
                         }
 
-                        colorValue=MyMath::linear_regression(minimaLog,maximaLog,
-                                                                                                255,0,max);
+                        colorValue=MyMath::linear_regression(minimaLog,maximaLog,255,0,max);
                         colorValue=colorValue>255?255:colorValue<0?0:colorValue;
                         //color.setRgb(colorValue,colorValue,colorValue);
                         color.setHsv(colorValue,255,255);
@@ -537,12 +639,13 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
                         */
 
                     }
-
+/*
                     if((i-start_sample)%(jump*100)==0){
                         progress.setProgress(i-start_sample);
                         progress.setLabelText("Drawing spectrum...(frame: "+QString::number(i)+")");
+                        mainApplication->processEvents();
                     }
-
+*/
             }
             if(lastchunk>0){
                 localpainter.fillRect(x,0,v_width-c_x-x,v_height,QBrush(blue));
@@ -588,7 +691,7 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
 /*!
     \fn WaveCanvas::setPixmapXOrigin(int)
 
-    sets the origin of the pixmap
+    sets the origin of the pixmap. Used when repainting only the part of the image.
     \sa pixmapXOrigin
  */
 void WaveCanvas::setPixmapXOrigin(int x_orig)
@@ -648,6 +751,7 @@ void WaveCanvas::zapBuffers(long start, long end)
 
 /*!
     \fn WaveCanvas::setVisibleWidth(int w)
+    Sets the with of the displayed part of the canvas.
  */
 void WaveCanvas::setVisibleWidth(int w)
 {
@@ -657,7 +761,9 @@ void WaveCanvas::setVisibleWidth(int w)
 
 /*!
     \fn WaveCanvas::emptyBuffers()
+    Flushes the buffers containing max and min values for amplitude and spectrogram drawing.
  */
+
 void WaveCanvas::emptyBuffers()
 {
 
@@ -669,3 +775,8 @@ void WaveCanvas::emptyBuffers()
             spectrogramMaxBuffer[i].hasValue=FALSE;
     }
 }
+
+
+void WaveCanvas::setMainApplication(QApplication* theValue) {
+    mainApplication = theValue;
+}
